# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: app/app_logs_payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.app import app_definition_pb2 as app_dot_app__definition__pb2
from union.internal.app import replica_definition_pb2 as app_dot_replica__definition__pb2
from union.internal.logs.dataplane import payload_pb2 as logs_dot_dataplane_dot_payload__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61pp/app_logs_payload.proto\x12\x0c\x63loudidl.app\x1a\x18\x61pp/app_definition.proto\x1a\x1c\x61pp/replica_definition.proto\x1a\x1clogs/dataplane/payload.proto\x1a\x17validate/validate.proto\"\xa9\x01\n\x0fTailLogsRequest\x12;\n\x06\x61pp_id\x18\x01 \x01(\x0b\x32\x18.cloudidl.app.IdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x05\x61ppId\x12J\n\nreplica_id\x18\x02 \x01(\x0b\x32\x1f.cloudidl.app.ReplicaIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\treplicaIdB\r\n\x06target\x12\x03\xf8\x42\x01\"T\n\x15ReplicaIdentifierList\x12;\n\x08replicas\x18\x01 \x03(\x0b\x32\x1f.cloudidl.app.ReplicaIdentifierR\x08replicas\"\xbb\x01\n\x08LogLines\x12\x18\n\x05lines\x18\x01 \x03(\tB\x02\x18\x01R\x05lines\x12H\n\nreplica_id\x18\x02 \x01(\x0b\x32\x1f.cloudidl.app.ReplicaIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\treplicaId\x12K\n\x10structured_lines\x18\x03 \x03(\x0b\x32 .cloudidl.logs.dataplane.LogLineR\x0fstructuredLines\";\n\rLogLinesBatch\x12*\n\x04logs\x18\x01 \x03(\x0b\x32\x16.cloudidl.app.LogLinesR\x04logs\"\xfd\x01\n\x10TailLogsResponse\x12K\n\x08replicas\x18\x01 \x01(\x0b\x32#.cloudidl.app.ReplicaIdentifierListB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x08replicas\x12L\n\tlog_lines\x18\x02 \x01(\x0b\x32!.cloudidl.logs.dataplane.LogLinesB\n\x18\x01\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x08logLines\x12\x41\n\x07\x62\x61tches\x18\x03 \x01(\x0b\x32\x1b.cloudidl.app.LogLinesBatchB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x07\x62\x61tchesB\x0b\n\x04resp\x12\x03\xf8\x42\x01\x42\xa2\x01\n\x10\x63om.cloudidl.appB\x13\x41ppLogsPayloadProtoH\x02P\x01Z&github.com/unionai/cloud/gen/pb-go/app\xa2\x02\x03\x43\x41X\xaa\x02\x0c\x43loudidl.App\xca\x02\x0c\x43loudidl\\App\xe2\x02\x18\x43loudidl\\App\\GPBMetadata\xea\x02\rCloudidl::Appb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'app.app_logs_payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020com.cloudidl.appB\023AppLogsPayloadProtoH\002P\001Z&github.com/unionai/cloud/gen/pb-go/app\242\002\003CAX\252\002\014Cloudidl.App\312\002\014Cloudidl\\App\342\002\030Cloudidl\\App\\GPBMetadata\352\002\rCloudidl::App'
  _TAILLOGSREQUEST.oneofs_by_name['target']._options = None
  _TAILLOGSREQUEST.oneofs_by_name['target']._serialized_options = b'\370B\001'
  _TAILLOGSREQUEST.fields_by_name['app_id']._options = None
  _TAILLOGSREQUEST.fields_by_name['app_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILLOGSREQUEST.fields_by_name['replica_id']._options = None
  _TAILLOGSREQUEST.fields_by_name['replica_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LOGLINES.fields_by_name['lines']._options = None
  _LOGLINES.fields_by_name['lines']._serialized_options = b'\030\001'
  _LOGLINES.fields_by_name['replica_id']._options = None
  _LOGLINES.fields_by_name['replica_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILLOGSRESPONSE.oneofs_by_name['resp']._options = None
  _TAILLOGSRESPONSE.oneofs_by_name['resp']._serialized_options = b'\370B\001'
  _TAILLOGSRESPONSE.fields_by_name['replicas']._options = None
  _TAILLOGSRESPONSE.fields_by_name['replicas']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILLOGSRESPONSE.fields_by_name['log_lines']._options = None
  _TAILLOGSRESPONSE.fields_by_name['log_lines']._serialized_options = b'\030\001\372B\005\212\001\002\020\001'
  _TAILLOGSRESPONSE.fields_by_name['batches']._options = None
  _TAILLOGSRESPONSE.fields_by_name['batches']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TAILLOGSREQUEST']._serialized_start=156
  _globals['_TAILLOGSREQUEST']._serialized_end=325
  _globals['_REPLICAIDENTIFIERLIST']._serialized_start=327
  _globals['_REPLICAIDENTIFIERLIST']._serialized_end=411
  _globals['_LOGLINES']._serialized_start=414
  _globals['_LOGLINES']._serialized_end=601
  _globals['_LOGLINESBATCH']._serialized_start=603
  _globals['_LOGLINESBATCH']._serialized_end=662
  _globals['_TAILLOGSRESPONSE']._serialized_start=665
  _globals['_TAILLOGSRESPONSE']._serialized_end=918
# @@protoc_insertion_point(module_scope)



import os
from dataclasses import dataclass
from typing import Optional

import grpc
from flytekit.clients.auth_helper import (
    RemoteClientConfigStore,
    get_authenticator,
)
from flytekit.configuration import Config, PlatformConfig

from union._config import _get_authenticated_channel
from union.filesystems._middleware import (
    StreamStreamClientServiceAccountMetadataInterceptor,
    StreamUnaryClientServiceAccountMetadataInterceptor,
    UnaryStreamClientAuthInterceptor,
    UnaryStreamClientServiceAccountMetadataInterceptor,
    UnaryUnaryClientAuthInterceptor,
    UnaryUnaryClientServiceAccountMetadataInterceptor,
)

_ENDPOINT = "localhost:8080"
MAX_MESSAGE_LENGTH = 1 * 1024 * 1024 * 1024
COMMON_GRPC_OPTIONS = [
    ("grpc.max_message_length", MAX_MESSAGE_LENGTH),
    ("grpc.max_send_message_length", MAX_MESSAGE_LENGTH),
    ("grpc.max_receive_message_length", MAX_MESSAGE_LENGTH),
]


def _get_endpoint() -> str:
    endpoint = os.environ.get("OBJECT_STORE_ENDPOINT")
    if endpoint is not None and endpoint != "":
        return endpoint

    endpoint = os.environ.get("object_store_endpoint")
    if endpoint is not None and endpoint != "":
        return endpoint

    return _ENDPOINT


def _is_remote(endpoint: str) -> bool:
    return endpoint != _ENDPOINT


def _create_channel_for_execution_environment():
    endpoint = _get_endpoint()

    interceptors = None
    if _is_remote(endpoint):
        unary_unary_auth_interceptor = UnaryUnaryClientServiceAccountMetadataInterceptor()
        unary_stream_auth_interceptor = UnaryStreamClientServiceAccountMetadataInterceptor()
        stream_unary_auth_interceptor = StreamUnaryClientServiceAccountMetadataInterceptor()
        stream_stream_auth_interceptor = StreamStreamClientServiceAccountMetadataInterceptor()
        interceptors = [
            unary_unary_auth_interceptor,
            unary_stream_auth_interceptor,
            stream_unary_auth_interceptor,
            stream_stream_auth_interceptor,
        ]

    channel = grpc.aio.insecure_channel(
        endpoint,
        options=COMMON_GRPC_OPTIONS,
        interceptors=interceptors,
    )
    return channel


def _create_secure_channel_from_config(
    platform_cfg: PlatformConfig, sync_channel: Optional[grpc.Channel] = None
) -> grpc.aio.Channel:
    # Add authenticator interceptor that uses the original channel. Use a sync channel
    # here to get the credentials.
    if sync_channel is None:
        sync_channel = _get_authenticated_channel(platform_cfg)

    authenticator = get_authenticator(platform_cfg, RemoteClientConfigStore(sync_channel))
    if authenticator.get_credentials() is None:
        authenticator.refresh_credentials()

    return grpc.aio.secure_channel(
        platform_cfg.endpoint,
        credentials=grpc.ssl_channel_credentials(),
        options=COMMON_GRPC_OPTIONS,
        interceptors=[
            UnaryUnaryClientAuthInterceptor(authenticator),
            UnaryStreamClientAuthInterceptor(authenticator),
        ],
    )


@dataclass
class RemoteConnection:
    channel: grpc.aio._channel
    in_execution_env: bool


def _create_channel(config: Optional[Config] = None) -> RemoteConnection:
    endpoint = _get_endpoint()
    if _is_remote(endpoint) or config is None:
        return RemoteConnection(channel=_create_channel_for_execution_environment(), in_execution_env=True)
    return RemoteConnection(_create_secure_channel_from_config(config.platform), in_execution_env=False)



from __future__ import annotations

import json
import logging
import os
import shutil
import tempfile
from dataclasses import dataclass
from datetime import datetime
from typing import Dict, Optional, Tuple, Union

from flytekit import ImageSpec, Resources, Secret, Workflow, current_context
from flytekit.core.context_manager import ExecutionParameters
from flytekit.extras import accelerators

import union
from union.artifacts import Artifact
from union.artifacts._card import ModelCard
from union.remote import UnionRemote
from union.remote._app_template_factory import (
    ARCHITECTURE_KEY,
    ARTIFACT_TYPE_KEY,
    COMMIT_KEY,
    FORMAT_KEY,
    HUGGINGFACE_SOURCE_KEY,
    MODALITY_KEY,
    MODEL_TYPE_KEY,
    SHARD_ENGINE_KEY,
    SHARD_PARALLELISM_KEY,
    TASK_KEY,
    HuggingFaceModelInfo,
    ShardConfig,
)

logger = logging.getLogger(__name__)


def _get_remote(ctx: ExecutionParameters) -> UnionRemote:
    """
    Get the remote object for the current execution. This is used to interact with the Union backend.
    Args:
        self: flytekit.core.context_manager.ExecutionParameters
    Returns: UnionRemote
    """
    project = ctx.execution_id.project if ctx.execution_id else None
    domain = ctx.execution_id.domain if ctx.execution_id else None
    raw_output = ctx.raw_output_prefix
    return UnionRemote(config=None, project=project, domain=domain, data_upload_location=raw_output)


def _emit_artifact(ctx: ExecutionParameters, o: Artifact) -> Artifact:
    """
    Emit an artifact to Union. This will create a new artifact with the given name and version and will
    associate with this execution.
    If o is None or not an Artifact, this function will do nothing.
    Args:
        self: flytekit.core.context_manager.ExecutionParameters
        o: Artifact

    Raises: Exception if artifact creation fails.
    """
    # TODO add node_id to the context.
    from union.internal.artifacts import artifacts_pb2

    # Emit artifact
    if "HOSTNAME" in os.environ:
        hostname = os.environ["HOSTNAME"]
        try:
            node_id = hostname.split("-")[1]
        except Exception:
            node_id = "n1"
    else:
        node_id = "n1"

    o.set_source(
        artifacts_pb2.ArtifactSource(
            workflow_execution=ctx.execution_id.to_flyte_idl(),
            task_id=ctx.task_id.to_flyte_idl(),
            retry_attempt=int(os.getenv("FLYTE_ATTEMPT_NUMBER", "0")),
            node_id=node_id,
        )
    )
    remote = _get_remote(ctx)
    try:
        return remote.create_artifact(o)
    except Exception as e:
        logger.error(f"Failed to create artifact {o}: {e}")
        return remote.get_artifact(query=o.query().to_flyte_idl())


def lookup_huggingface_model_info(model_repo: str, commit: str, token: str) -> Tuple[str, str]:
    """
    Lookup Hugging Face model info for a model repo.
    This looks up the model info in huggingface config.json file.

    The assumed path is of the kind:
        https://huggingface.co/{model_repo}/resolve/{commit}/config.json
    example:
        https://huggingface.co/deepseek-ai/DeepSeek-V3-Base/resolve/69cf1d97df36843c038062eed5672df1d8480b32/config.json
    The json file is downloaded and the following fields are extracted:
        - model_type
        - architecture
    :param model_repo: The model repo name in huggingface
    :param token: The huggingface token for private models
    :return: HuggingFaceModelInfo
    """

    from huggingface_hub import hf_hub_download

    config_file = hf_hub_download(repo_id=model_repo, filename="config.json", revision=commit, token=token)
    arch = None
    model_type = None
    with open(config_file, "r") as f:
        j = json.load(f)
        arch = j.get("architecture", None)
        if arch is None:
            arch = j.get("architectures", None)
            if arch:
                arch = ",".join(arch)
            model_type = j.get("model_type", None)
    return model_type, arch


def get_partition_keys_for_model(info: HuggingFaceModelInfo, commit: str) -> Dict[str, str]:
    """
    Get partition keys for a model, given the architecture, task, modality, serial format and model type.

    :param info: The model info
    :return: The partition keys for the model
    """
    return {
        ARCHITECTURE_KEY: info.architecture,
        TASK_KEY: info.task,
        FORMAT_KEY: info.serial_format,
        HUGGINGFACE_SOURCE_KEY: info.repo,
        COMMIT_KEY: commit,
        MODALITY_KEY: ",".join(info.modality),
        ARTIFACT_TYPE_KEY: "model",
        MODEL_TYPE_KEY: info.model_type,
        SHARD_ENGINE_KEY: str(info.shard_config.engine) if info.shard_config else "None",
        SHARD_PARALLELISM_KEY: str(info.shard_config.args.tensor_parallel_size) if info.shard_config else "None",
    }


def download_all_files_to_flytedir(
    repo_id: str,
    commit: str,
    token: str | None = None,
) -> Tuple[union.FlyteDirectory, str | None]:
    """
    TODO we should use hf-transfer for this, but the only option on hf-transfer is to download the files to local disk.
    Stream all files in a Hugging Face Hub repository to a FlyteDirectory.

    Args:
        :param repo_id: str The repository ID (e.g., 'julien-c/EsperBERTo-small').
        :param commit: str The commit ID.
        :param token: str[optional] The Hugging Face Hub token for authentication.
    """
    from huggingface_hub import HfFileSystem, snapshot_download

    directory = union.FlyteDirectory.new("model_snapshot")
    card = None

    hfs = HfFileSystem(token=token)

    try:
        readme_file_details = hfs.info(f"{repo_id}/README.md", revision=commit)
        readme_name = readme_file_details["name"]
        with tempfile.NamedTemporaryFile() as temp_file:
            hfs.download(readme_name, temp_file.name, revision=commit)
            with open(temp_file.name, "r") as f:
                card = f.read()

    except FileNotFoundError:
        print("No readme file", flush=True)

    print(f"Downloading model from {repo_id} to {directory.path}", flush=True)
    snapshot_download(
        repo_id=repo_id,
        revision=commit,
        local_dir=directory.path,
        token=token,
    )
    return directory, card


def shard_model(
    shard_config: ShardConfig,
    model_path: str,
) -> union.FlyteDirectory:
    """Shard a model using the given shard config."""

    from vllm import LLM

    assert shard_config.engine == "vllm", "'vllm' is the only supported sharding engine for now"
    sharded_model_dir = union.FlyteDirectory.new("sharded_model_snapshot")

    # Create LLM instance from arguments
    llm = LLM(**shard_config.args.get_vllm_args(model_path))
    print(f"LLM initialized: {llm}")

    # Check which engine version is being used
    is_v1_engine = hasattr(llm.llm_engine, "engine_core")

    if is_v1_engine:
        # For V1 engine, we need to use engine_core.save_sharded_state
        print("Using V1 engine save path")
        llm.llm_engine.engine_core.save_sharded_state(
            path=sharded_model_dir.path,
            pattern=shard_config.args.file_pattern,
            max_size=shard_config.args.max_file_size,
        )
    else:
        # For V0 engine
        print("Using V0 engine save path")
        model_executor = llm.llm_engine.model_executor
        model_executor.save_sharded_state(
            path=sharded_model_dir.path,
            pattern=shard_config.args.file_pattern,
            max_size=shard_config.args.max_file_size,
        )

    # Copy metadata files to output directory
    print(f"Copying metadata files to {sharded_model_dir.path}")
    for file in os.listdir(model_path):
        if os.path.splitext(file)[1] not in (".bin", ".pt", ".safetensors"):
            if os.path.isdir(os.path.join(model_path, file)):
                shutil.copytree(os.path.join(model_path, file), os.path.join(sharded_model_dir.path, file))
            else:
                shutil.copy(os.path.join(model_path, file), sharded_model_dir.path)

    return sharded_model_dir


# Container and secrets are set in the create_hf_model_cache_workflow
@union.task
def validate_repo(info: HuggingFaceModelInfo, hf_token_key: str) -> Tuple[str, datetime]:
    """
    Validate if the repo exists in Hugging Face Hub.
    Args:
        info: HuggingFaceModelInfo: The model info.

    Returns:
        Returns the latest version of the model in the huggingface repo.
    """
    from huggingface_hub import list_repo_commits, repo_exists

    token = current_context().secrets.get(key=hf_token_key)
    if not repo_exists(info.repo, token=token):
        raise ValueError(f"Repository {info.repo} does not exist in huggingface.")

    commit = list_repo_commits(info.repo, token=token)[0]
    return commit.commit_id, commit.created_at


@dataclass
class ArtifactInfo:
    artifact_name: str
    blob: str
    model_uri: str


# Container, secrets, and resources are set in the create_hf_model_cache_workflow
@union.task(cache=True, cache_version="1.1")
def cache_model_from_hf(info: HuggingFaceModelInfo, commit: str, retry: int, hf_token_key: str) -> ArtifactInfo:
    """
    This task caches a model from the Hugging Face Hub, given the model info.
    Args:
        info: HuggingFaceModelInfo: The model info.
        commit: str: The commit id of the model.

    Returns:
        FlyteDirectory: The model artifact.
    """
    print("Model info: ", info)
    print(f"Caching model from huggingface repo: {info.repo}, commit: {commit}", flush=True)
    ctx = union.current_context()
    token = ctx.secrets.get(key=hf_token_key)
    if not info.model_type or not info.architecture:
        print("Looking up huggingface model info...")
        model_type = "custom"
        architecture = "custom"
        try:
            model_type, architecture = lookup_huggingface_model_info(info.repo, commit, token)
        except Exception as e:
            print(f"Error looking up huggingface model info: {e}")
        info.model_type = info.model_type or model_type
        info.architecture = info.architecture or architecture

    print(f"Model type: {info.model_type}, architecture: {info.architecture}")

    partitions = get_partition_keys_for_model(info, commit)
    print(f"Partitions: {partitions}")

    print("Downloading files...", flush=True)
    directory, card = download_all_files_to_flytedir(info.repo, commit, token)
    print(f"Data downloaded to {directory.path}")

    if info.shard_config is not None:
        print(f"Sharding model with {info.shard_config.engine} engine")
        directory = shard_model(info.shard_config, directory.path)

    if info.artifact_name is None:
        artifact_name = info.repo.split("/")[-1]
        artifact_name = artifact_name.replace(".", "-")
    else:
        artifact_name = info.artifact_name

    o = union.Artifact(
        name=artifact_name,
        python_type=union.FlyteDirectory,
        python_val=directory,
        short_description=f"Model cached from huggingface repo: {info.repo}, commit: {commit} "
        f"by execution: {ctx.execution_id}.",
        partitions=partitions,
        project=ctx.execution_id.project if ctx.execution_id else None,
        domain=ctx.execution_id.domain if ctx.execution_id else None,
        card=ModelCard.from_obj(card) if card else None,
    )
    print(f"Emitting artifact, {o}")
    a: union.Artifact = _emit_artifact(ctx, o)
    print(f"Artifact emitted, {a.metadata().uri}")
    return ArtifactInfo(artifact_name=artifact_name, blob=directory.path, model_uri=a.metadata().uri if a else "NA")


def create_hf_model_cache_workflow(
    image: Union[str, ImageSpec],
    hf_token_key: str,
    union_api_key: str,
    resources: Optional[Resources] = None,
    accelerator: Optional[str] = None,
):
    """
    Create workflow runs the cache_model_from_hf task.

    The arguments are:
    image: image to run tasks in.
    retry: this can be used to force a new artifact to be created with the same name and an incremented version,
            this will create a new copy in blob store too
    info: HuggingFaceInfo: The model info.

    The outputs are:
    ArtifactInfo: The model artifact
    """
    imperative_wf = Workflow(name=f"{__name__}.hf_model_cacher")
    imperative_wf.add_workflow_input("info", HuggingFaceModelInfo)
    imperative_wf.add_workflow_input("retry", int)
    imperative_wf.add_workflow_input("hf_token_key", str)

    if resources is None:
        resources = union.Resources(mem="2Gi", cpu="2", ephemeral_storage="16Gi")

    task_kwargs = {}
    if accelerator is not None:
        if accelerator == "nvidia-a100":
            accelerator = accelerators.A100
        elif accelerator == "nvidia-a100-80gb":
            accelerator = accelerators.A100_80GB
        else:
            accelerator = accelerators.GPUAccelerator(accelerator)
        task_kwargs["accelerator"] = accelerator
        task_kwargs["shared_memory"] = True

    hf_secret = Secret(key=hf_token_key)
    union_api_secret = Secret(key=union_api_key, env_var="UNION_API_KEY")
    additional_context = [
        str(resources.to_json()),
        str(hf_secret.serialize_to_string()),
        str(union_api_secret.serialize_to_string()),
    ]

    validate_repo_task = union.task(
        container_image=image,
        requests=resources,
        limits=resources,
        secret_requests=[hf_secret],
        **task_kwargs,
    )(validate_repo.task_function)

    validate_repo_node = imperative_wf.add_entity(
        validate_repo_task,
        info=imperative_wf.inputs["info"],
        hf_token_key=imperative_wf.inputs["hf_token_key"],
    )

    cache_mode_task = union.task(
        container_image=image,
        requests=resources,
        limits=resources,
        secret_requests=[hf_secret, union_api_secret],
        **task_kwargs,
    )(cache_model_from_hf.task_function)

    cache_model_from_hf_node = imperative_wf.add_entity(
        cache_mode_task,
        info=imperative_wf.inputs["info"],
        commit=validate_repo_node.outputs["o0"],
        retry=imperative_wf.inputs["retry"],
        hf_token_key=imperative_wf.inputs["hf_token_key"],
    )
    imperative_wf.add_workflow_output("artifact", cache_model_from_hf_node.outputs["o0"])

    return imperative_wf, os.path.dirname(union.__path__[0]), additional_context



from click import Argument, Option, UsageError


class MutuallyExclusiveMixin:
    def __init__(self, *args, **kwargs):
        self.mutually_exclusive = set(kwargs.pop("mutually_exclusive", []))
        self.error_format = kwargs.pop("error_msg", "Illegal usage: {name} is mutually exclusive with {invalid}")
        super().__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        self_present = self.name in opts and opts[self.name] is not None
        others_intersect = self.mutually_exclusive.intersection(opts)
        others_present = others_intersect and any(opts[value] is not None for value in others_intersect)

        if others_present:
            if self_present:
                raise UsageError(self.error_format.format(name=self.name, invalid=", ".join(self.mutually_exclusive)))
            else:
                self.prompt = None

        return super().handle_parse_result(ctx, opts, args)


# See https://stackoverflow.com/a/37491504/499285 and https://stackoverflow.com/a/44349292/499285
class MutuallyExclusiveOption(MutuallyExclusiveMixin, Option):
    def __init__(self, *args, **kwargs):
        mutually_exclusive = kwargs.get("mutually_exclusive", [])
        help = kwargs.get("help", "")
        if mutually_exclusive:
            kwargs["help"] = help + f" Mutually exclusive with {', '.join(mutually_exclusive)}."
        super().__init__(*args, **kwargs)


class MutuallyExclusiveArgument(MutuallyExclusiveMixin, Argument):
    pass



import os

from flytekit import (
    ContainerTask,
    Deck,
    ImageSpec,
    LaunchPlan,
    PodTemplate,
    Resources,
    Secret,
    StructuredDataset,
    current_context,
    dynamic,
    map_task,
    task,
    workflow,
)
from flytekit.core.cache import Cache, CachePolicy, VersionParameters
from flytekit.types.directory import FlyteDirectory
from flytekit.types.file import FlyteFile

from union._logging import _init_global_loggers
from union.actor import ActorEnvironment, actor_cache
from union.artifacts import Artifact
from union.map import map
from union.remote import UnionRemote

# Set GRPC_VERBOSITY to NONE if not already set to silence unwanted output
# This addresses the issue with grpcio >=1.68.0 causing unwanted output
if "GRPC_VERBOSITY" not in os.environ:
    os.environ["GRPC_VERBOSITY"] = "NONE"

_init_global_loggers()


__all__ = [
    "ActorEnvironment",
    "Artifact",
    "Cache",
    "CachePolicy",
    "ContainerTask",
    "Deck",
    "FlyteDirectory",
    "FlyteFile",
    "ImageSpec",
    "LaunchPlan",
    "PodTemplate",
    "Resources",
    "Secret",
    "StructuredDataset",
    "UnionRemote",
    "VersionParameters",
    "actor_cache",
    "current_context",
    "dynamic",
    "map",
    "map_task",
    "task",
    "workflow",
]



import shlex
from dataclasses import dataclass
from typing import List, Optional, Union

from flytekit.core.artifact import ArtifactQuery
from flytekit.core.pod_template import PodTemplate

from union import ImageSpec
from union.app import App, Input

FLASH_INFER_INDEX_URL = "https://flashinfer.ai/whl/cu124/torch2.5"
DEFAULT_SGLANG_IMAGE = "ghcr.io/unionai/serving-sglang:py3.12-latest"
OPTIMIZED_SGLANG_IMAGE = "managed.cr.union.ai/sglang:stable"


@dataclass
class SGLangApp(App):
    """
    App backed by FastAPI.

    :param name: The name of the application.
    :param container_image: The container image to use for the application.
    :param args: Entrypoint to start application.
    :param command: Command to start application.
    :param port: Port application listens to.
    :param requests: Compute resource requests for application.
    :param secrets: Secrets that are requested for application.
    :param limits: Compute resource limits for application.
    :param include: Files to include for your application.
    :param inputs: Inputs for the application.
    :param env: Environment variables to set for the application.
    :param min_replicas: Minimum number of replicas (ignore if autoscaling is set).
    :param max_replicas: Maximum number of replicas (ignore if autoscaling is set).
    :param scaledown_after: Time to wait before scaling down a replica after it has been idle.
    :param scaling_metric: Autoscale based on a parameter, e.g. request rate or concurrency
      (others may be added in the future).
    :param cluster_pool: The target cluster_pool where the app should be deployed.
    :param requires_auth: Public URL does not require any authentication
    :param type: Type of app
    :param description: Description of app
    :param subdomain: Custom subdomain for your app.
    :param custom_domain: Custom full domain for your app.
    :param extra_args: Extra args to pass to `python -m sglang.launch_server`. See
        https://docs.sglang.ai/backend/server_arguments.html for details.
    :param model: Artifact URI for model.
        - If `str`, it should start with `flyte://`.
        - Use `ArtifactQuery` to dynamically query an artifact.
    :param model_id: model id that is exposed by vllm.
    :param stream_model: Set to True to stream model from blob store to the GPU directly.
      If False, the model will be downloaded to the local file system first and then loaded
      into the GPU.
    """

    port: int = 8000
    type: str = "SGLang"
    container_image: Optional[Union[str, ImageSpec, PodTemplate]] = None
    extra_args: Union[str, List[str]] = ""
    model: Union[str, ArtifactQuery] = ""
    model_id: str = ""
    stream_model: bool = True

    def __post_init__(self):
        if self.model_id == "":
            raise ValueError("model_id must be defined")

        if self.args:
            raise ValueError("args can not be set for SGLangApp. Use `extra_args` to add extra arguments to SGLang")

        if isinstance(self.extra_args, str):
            extra_args = shlex.split(self.extra_args)
        else:
            extra_args = self.extra_args

        self.args = [
            "union-model-loader-sglang",
            "--model-path",
            "/root/union",
            "--served-model-name",
            self.model_id,
            "--port",
            str(self.port),
            *extra_args,
        ]

        if self.inputs:
            raise ValueError("inputs can not be set for SGLangApp")

        if isinstance(self.model, str):
            input_type = Input.Type._ArtifactUri
        elif isinstance(self.model, ArtifactQuery):
            input_type = None
        else:
            msg = "model must be a string or ArtifactQuery"
            raise TypeError(msg)

        if self.container_image is None:
            self.container_image = DEFAULT_SGLANG_IMAGE
        elif isinstance(self.container_image, ImageSpec):
            # NOTE: the flashinfer index url has to be included because the
            # union builder fails to install the nvidia drivers needed to
            # install flashinfer.
            if (
                self.container_image.pip_extra_index_url
                and FLASH_INFER_INDEX_URL not in self.container_image.pip_extra_index_url
            ):
                self.container_image.pip_extra_index_url.append(FLASH_INFER_INDEX_URL)
            elif self.container_image.pip_extra_index_url is None:
                self.container_image.pip_extra_index_url = [FLASH_INFER_INDEX_URL]

        input_kwargs = {}
        if self.stream_model:
            self.env["UNION_MODEL_LOADER_STREAM_SAFETENSORS"] = "true"
            input_kwargs["env_var"] = "UNION_MODEL_LOADER_REMOTE_MODEL_PATH"
            input_kwargs["download"] = False
        else:
            self.env["UNION_MODEL_LOADER_STREAM_SAFETENSORS"] = "false"
            input_kwargs["download"] = True
            input_kwargs["mount"] = "/root/union"

        self.inputs = [Input(name="model", value=self.model, type=input_type, **input_kwargs)]

        self.env["UNION_MODEL_LOADER_LOCAL_MODEL_PATH"] = "/root/union"

        super().__post_init__()



from union.internal.app import app_logs_payload_pb2 as _app_logs_payload_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



from base64 import decodebytes, encodebytes
from configparser import ConfigParser, NoOptionError, NoSectionError
from pathlib import Path
from typing import Optional

from keyring.backend import KeyringBackend
from keyring.errors import PasswordDeleteError

_UNION_KEYRING_PATH: Path = Path.home() / ".union" / "keyring.cfg"


class SimplePlainTextKeyring(KeyringBackend):
    """Simple plain text keyring"""

    priority = 0.5

    def get_password(self, service: str, username: str) -> Optional[str]:
        """Get password."""
        if not self.file_path.exists():
            return None

        config = ConfigParser(interpolation=None)
        config.read(self.file_path, encoding="utf-8")

        try:
            password_base64 = config.get(service, username).encode("utf-8")
            return decodebytes(password_base64).decode("utf-8")
        except (NoOptionError, NoSectionError):
            return None

    def delete_password(self, service: str, username: str) -> None:
        """Delete password."""
        if not self.file_path.exists():
            raise PasswordDeleteError("Config file does not exist")

        config = ConfigParser(interpolation=None)
        config.read(self.file_path, encoding="utf-8")

        try:
            if not config.remove_option(service, username):
                raise PasswordDeleteError("Password not found")
        except NoSectionError:
            raise PasswordDeleteError("Password not found")

        with self.file_path.open("w", encoding="utf-8") as config_file:
            config.write(config_file)

    def set_password(self, service: str, username: str, password: str) -> None:
        """Set password."""
        if not username:
            raise ValueError("Username must be provided")

        file_path = self._ensure_file_path()
        value = encodebytes(password.encode("utf-8")).decode("utf-8")

        config = ConfigParser(interpolation=None)
        config.read(file_path, encoding="utf-8")

        if not config.has_section(service):
            config.add_section(service)

        config.set(service, username, value)

        with file_path.open("w", encoding="utf-8") as config_file:
            config.write(config_file)

    def _ensure_file_path(self):
        self.file_path.parent.mkdir(exist_ok=True, parents=True)
        if not self.file_path.is_file():
            self.file_path.touch(0o600)
        return self.file_path

    @property
    def file_path(self) -> Path:
        return _UNION_KEYRING_PATH

    def __repr__(self):
        return f"<{self.__class__.__name__}> at {self.file_path}>"



import asyncio
import datetime
import logging
import random
import string
import weakref
from typing import AsyncGenerator, Optional

import grpc
from flyteidl.core import security_pb2 as _security_pb2
from flyteidl.core import tasks_pb2 as _tasks_pb2
from flytekit import WorkflowExecutionPhase
from flytekit.configuration import Config
from flytekit.models.core.execution import TaskExecutionPhase
from flytekit.models.execution import Execution
from flytekit.models.filters import Equal, ValueIn

from union.cli._common import _get_channel_with_org
from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import list_pb2 as _list_pb2
from union.internal.workspace.workspace_definition_payload_pb2 import (
    CreateWorkspaceDefinitionRequest,
    CreateWorkspaceDefinitionResponse,
    GetWorkspaceDefinitionRequest,
    GetWorkspaceDefinitionResponse,
    ListWorkspaceDefinitionsRequest,
    ListWorkspaceDefinitionsResponse,
)
from union.internal.workspace.workspace_definition_pb2 import (
    WorkspaceDefinitionIdentifier,
    WorkspaceDefinitionSpec,
    WorkspaceDefinitionState,
)
from union.internal.workspace.workspace_definition_service_pb2_grpc import WorkspaceRegistryServiceStub
from union.internal.workspace.workspace_instance_payload_pb2 import (
    GetWorkspaceInstanceRequest,
    GetWorkspaceInstanceResponse,
    ListWorkspaceInstancesRequest,
    ListWorkspaceInstancesResponse,
    StartWorkspaceInstanceRequest,
    StartWorkspaceInstanceResponse,
    StopWorkspaceInstanceRequest,
    WatchWorkspaceInstancesRequest,
    WatchWorkspaceInstancesResponse,
)
from union.internal.workspace.workspace_instance_pb2 import (
    WorkspaceInstanceIdentifier,
    WorkspaceInstanceStatus,
)
from union.internal.workspace.workspace_instance_service_pb2_grpc import WorkspaceInstanceServiceStub
from union.remote import UnionRemote
from union.workspace._config import WorkspaceConfig

VERSION_STRING_LENGTH = 20
VSCODE_DEBUGGER_LOG_NAME = "VS Code Debugger"

_logger = logging.getLogger(__name__)


class WorkspaceRemote:
    def __init__(
        self,
        default_project: str,
        default_domain: str,
        config: Optional[Config] = None,
        union_remote: Optional[UnionRemote] = None,
    ):
        if union_remote is None:
            union_remote = UnionRemote(config=config, default_domain=default_domain, default_project=default_project)
            self._union_remote = union_remote
            self._union_remote_ref = weakref.ref(self._union_remote)
        else:
            # Union remote is passed in directly. We assume that the caller will have a reference to `AppRemote`.
            self._union_remote_ref = weakref.ref(union_remote)

        self.config = union_remote.config
        self.default_project = union_remote.default_project
        self.default_domain = union_remote.default_domain

    @property
    def union_remote(self) -> UnionRemote:
        union_remote = self._union_remote_ref()
        if union_remote is None:
            raise ValueError("Unable to find union remote")
        return union_remote

    def get_workspace_definition_id(
        self,
        workspace_definition_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
        version: Optional[str] = None,
    ) -> WorkspaceDefinitionIdentifier:
        return WorkspaceDefinitionIdentifier(
            org=self.org,
            project=project or self.default_project,
            domain=domain or self.default_domain,
            name=workspace_definition_name,
            version=version or f"{workspace_definition_name}-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}",
        )

    def get_workspace_instance_id(
        self,
        workspace_instance_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ) -> WorkspaceInstanceIdentifier:
        return WorkspaceInstanceIdentifier(
            org=self.org,
            project=project or self.default_project,
            domain=domain or self.default_domain,
            name=workspace_instance_name,
            host=self.org,
        )

    async def workspace_instance_is_stopping(
        self,
        workspace_instance_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
        logger: Optional[logging.Logger] = _logger,
    ) -> bool:
        """
        Check if the workspace instance is terminated or terminating.
        """
        workspace_instance_id = self.get_workspace_instance_id(
            workspace_instance_name=workspace_instance_name,
            project=project or self.default_project,
            domain=domain or self.default_domain,
        )
        logger.info(f"GetWorkspaceInstance {workspace_instance_id}")
        response: GetWorkspaceInstanceResponse = await self.instance_async_client.GetWorkspaceInstance(
            GetWorkspaceInstanceRequest(id=workspace_instance_id)
        )
        return response.workspace_instance.spec.status.phase in (
            WorkspaceInstanceStatus.WorkspaceInstancePhase.WORKSPACE_INSTANCE_PHASE_TERMINATING,
            WorkspaceInstanceStatus.WorkspaceInstancePhase.WORKSPACE_INSTANCE_PHASE_TERMINATED,
        )

    async def watch_workspace_instance(
        self,
        workspace_instance_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
        logger: Optional[logging.Logger] = _logger,
    ) -> AsyncGenerator[bool, None]:
        """
        Watch the workspace instance for stopping event.
        """

        logger.info(f"Watching workspace instance {workspace_instance_name} in project {project} and domain {domain}")

        workspace_instance_id = self.get_workspace_instance_id(
            workspace_instance_name=workspace_instance_name,
            project=project or self.default_project,
            domain=domain or self.default_domain,
        )
        watch_request = WatchWorkspaceInstancesRequest(workspace_instance_id=workspace_instance_id)

        response: WatchWorkspaceInstancesResponse
        logger.info(f"WatchWorkspaceInstances request {watch_request}")

        stream = self.instance_async_client.WatchWorkspaceInstances(watch_request)
        while True:
            try:
                async for response in stream:
                    logger.info(f"WatchWorkspaceInstances response {response}")
                    is_stopping = response.WhichOneof("event") == "stopping_event"
                    logger.info(f"Workspace instance is stopping: {is_stopping}")
                    yield is_stopping
                    if is_stopping:
                        return
            except Exception as exc:
                logger.info(f"Error watching workspace instance {exc}, resetting stream")
                stream = self.instance_async_client.WatchWorkspaceInstances(watch_request)
                continue

    async def _create_workspace_definition(
        self,
        workspace_config: WorkspaceConfig,
        version: Optional[str] = None,
        create_workspace_definition_entity: bool = True,
    ) -> CreateWorkspaceDefinitionResponse:
        """
        Create a workspace definition.

        :param workspace_config: The workspace config to create the workspace definition from.
        :param version: The version of the workspace definition.
        :param create_workspace_definition_entity: Whether to create the workspace definition entity.
        """
        workspace_definition_id = self.get_workspace_definition_id(
            workspace_definition_name=workspace_config.name,
            project=workspace_config.project,
            domain=workspace_config.domain,
            version=version,
        )

        kwargs = {}
        if workspace_config.resources:
            kwargs["resources"] = _tasks_pb2.Resources(
                requests=[
                    _tasks_pb2.Resources.ResourceEntry(name=name, value=value)
                    for name, value in (
                        (_tasks_pb2.Resources.ResourceName.CPU, workspace_config.resources.cpu),
                        (_tasks_pb2.Resources.ResourceName.MEMORY, workspace_config.resources.mem),
                        (_tasks_pb2.Resources.ResourceName.GPU, workspace_config.resources.gpu),
                        (
                            _tasks_pb2.Resources.ResourceName.EPHEMERAL_STORAGE,
                            workspace_config.resources.ephemeral_storage,
                        ),
                    )
                    if value is not None
                ],
            )
        if workspace_config.accelerator:
            kwargs["extended_resources"] = _tasks_pb2.ExtendedResources(
                gpu_accelerator=_tasks_pb2.GPUAccelerator(
                    device=workspace_config.accelerator,
                    unpartitioned=True,
                ),
            )
        if workspace_config.secrets:
            kwargs["security_context"] = _security_pb2.SecurityContext(
                secrets=[secret.to_flyte_idl() for secret in workspace_config.secrets]
            )

        workspace_definition_spec = WorkspaceDefinitionSpec(
            short_description=workspace_config.description,
            state=WorkspaceDefinitionState.WORKSPACE_DEFINITION_STATE_ACTIVE,
            ttl_seconds=workspace_config.ttl_seconds,
            on_startup=workspace_config.on_startup,
            container_image=workspace_config.container_image,
            workspace_root=workspace_config.workspace_root,
            working_dir=workspace_config.working_dir,
            **kwargs,
        )
        create_request = CreateWorkspaceDefinitionRequest(
            id=workspace_definition_id,
            workspace_spec=workspace_definition_spec,
            create_workspace_definition_entity=create_workspace_definition_entity,
        )
        try:
            response: CreateWorkspaceDefinitionResponse = await self.definition_async_client.CreateWorkspaceDefinition(
                create_request
            )
        except Exception as e:
            raise RuntimeError(f"Failed to create workspace definition: {e}") from e
        return response

    async def create_workspace_definition(
        self,
        workspace_config: WorkspaceConfig,
        version: Optional[str] = None,
    ) -> CreateWorkspaceDefinitionResponse:
        try:
            return await self._create_workspace_definition(
                workspace_config=workspace_config,
                version=version,
                create_workspace_definition_entity=True,
            )
        except Exception as e:
            raise RuntimeError(f"Failed to create workspace definition: {e}") from e

    async def update_workspace_definition(
        self,
        workspace_config: WorkspaceConfig,
        version: Optional[str] = None,
    ) -> CreateWorkspaceDefinitionResponse:
        try:
            return await self._create_workspace_definition(
                workspace_config=workspace_config,
                version=version,
                create_workspace_definition_entity=False,
            )
        except Exception as e:
            raise RuntimeError(f"Failed to update workspace definition: {e}") from e

    async def get_workspace_definition(
        self,
        workspace_definition_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ) -> GetWorkspaceDefinitionResponse:
        get_request = GetWorkspaceDefinitionRequest(
            id=WorkspaceDefinitionIdentifier(
                org=self.org,
                project=project or self.default_project,
                domain=domain or self.default_domain,
                name=workspace_definition_name,
            )
        )
        response: GetWorkspaceDefinitionResponse = await self.definition_async_client.GetWorkspaceDefinition(
            get_request
        )
        return response

    async def list_workspace_definitions(
        self,
        workspace_definition_name: Optional[str] = None,
        project: Optional[str] = None,
        domain: Optional[str] = None,
        sort_by: str = "created_at",
        direction: str = "desc",
    ) -> ListWorkspaceDefinitionsResponse:
        list_request = ListWorkspaceDefinitionsRequest(
            org=self.org,
            project=project or self.default_project,
            domain=domain or self.default_domain,
            name=workspace_definition_name,
            request=_list_pb2.ListRequest(
                limit=1000,
                sort_by=_list_pb2.Sort(
                    key=sort_by,
                    direction=(
                        _list_pb2.Sort.Direction.DESCENDING
                        if direction == "desc"
                        else _list_pb2.Sort.Direction.ASCENDING
                    ),
                ),
            ),
        )
        return await self.definition_async_client.ListWorkspaceDefinitions(list_request)

    async def get_latest_workspace_definition_id(
        self,
        workspace_definition_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ) -> WorkspaceDefinitionIdentifier:
        list_request = ListWorkspaceDefinitionsRequest(
            org=self.org,
            project=project or self.default_project,
            domain=domain or self.default_domain,
            name=workspace_definition_name,
            request=_list_pb2.ListRequest(
                limit=1,
                sort_by=_list_pb2.Sort(
                    key="created_at",
                    direction=_list_pb2.Sort.Direction.DESCENDING,
                ),
            ),
        )
        list_response: ListWorkspaceDefinitionsResponse = await self.definition_async_client.ListWorkspaceDefinitions(
            list_request
        )
        if len(list_response.workspace_definitions) == 0:
            raise RuntimeError(f"Workspace definition {workspace_definition_name} not found")
        return self.get_workspace_definition_id(
            workspace_definition_name=workspace_definition_name,
            project=project,
            domain=domain,
            version=list_response.workspace_definitions[0].id.version,
        )

    async def get_latest_workspace_instance_uri(
        self,
        workspace_definition_name: str,
        version: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ) -> str:
        workspace_definition_id = self.get_workspace_definition_id(
            workspace_definition_name=workspace_definition_name,
            project=project or self.default_project,
            domain=domain or self.default_domain,
            version=version,
        )
        list_request = ListWorkspaceInstancesRequest(
            request=_list_pb2.ListRequest(
                limit=1,
                sort_by=_list_pb2.Sort(
                    key="created_at",
                    direction=_list_pb2.Sort.Direction.DESCENDING,
                ),
            ),
            org=self.org,
            project=_identifier_pb2.ProjectIdentifier(
                organization=self.org,
                domain=domain or self.default_domain,
                name=project or self.default_project,
            ),
            workspace_definition_id=workspace_definition_id,
        )
        list_response: ListWorkspaceInstancesResponse = await self.instance_async_client.ListWorkspaceInstances(
            list_request
        )
        if len(list_response.workspace_instances) == 0:
            raise RuntimeError(f"Workspace instance for workspace '{workspace_definition_name}' not found")
        if (
            list_response.workspace_instances[0].spec.status.phase
            != WorkspaceInstanceStatus.WorkspaceInstancePhase.WORKSPACE_INSTANCE_PHASE_RUNNING
        ):
            return None
        return list_response.workspace_instances[0].spec.uri

    async def start_workspace_instance(
        self,
        workspace_definition_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ) -> GetWorkspaceInstanceResponse:
        workspace_definition_id = await self.get_latest_workspace_definition_id(
            workspace_definition_name=workspace_definition_name,
            project=project or self.default_project,
            domain=domain or self.default_domain,
        )
        _logger.info(f"WorkspaceDefinitionIdentifier: {workspace_definition_id}")
        start_request = StartWorkspaceInstanceRequest(
            org=workspace_definition_id.org,
            project=workspace_definition_id.project,
            domain=workspace_definition_id.domain,
            workspace_definition_id=workspace_definition_id,
        )
        _logger.info(f"StartWorkspaceInstanceRequest: {start_request}")
        try:
            start_response: StartWorkspaceInstanceResponse = await self.instance_async_client.StartWorkspaceInstance(
                start_request
            )
        except Exception:
            raise

        get_request = GetWorkspaceInstanceRequest(id=start_response.workspace_instance.id)
        uri = None
        _logger.info(f"GetWorkspaceInstanceRequest: {get_request}")
        while not uri:
            get_response: GetWorkspaceInstanceResponse = await self.instance_async_client.GetWorkspaceInstance(
                get_request
            )
            uri = get_response.workspace_instance.spec.uri

            if not uri:
                await asyncio.sleep(3)
        return get_response

    async def stop_workspace_instance(
        self,
        workspace_definition_name: str,
        project: Optional[str] = None,
        domain: Optional[str] = None,
    ):
        workspace_definition_id = await self.get_latest_workspace_definition_id(
            workspace_definition_name=workspace_definition_name,
            project=project or self.default_project,
            domain=domain or self.default_domain,
        )
        list_request = ListWorkspaceInstancesRequest(
            request=_list_pb2.ListRequest(
                limit=1,
                sort_by=_list_pb2.Sort(
                    key="created_at",
                    direction=_list_pb2.Sort.Direction.DESCENDING,
                ),
            ),
            org=self.org,
            project=_identifier_pb2.ProjectIdentifier(
                organization=self.org,
                domain=domain or self.default_domain,
                name=project or self.default_project,
            ),
            workspace_definition_id=workspace_definition_id,
        )
        list_response: ListWorkspaceInstancesResponse = await self.instance_async_client.ListWorkspaceInstances(
            list_request
        )
        if len(list_response.workspace_instances) == 0:
            raise RuntimeError(f"Workspace instance for workspace '{workspace_definition_name}' not found")

        workspace_instance_id = list_response.workspace_instances[0].id

        stop_request = StopWorkspaceInstanceRequest(id=workspace_instance_id)
        _logger.info(f"StopWorkspaceInstanceRequest: {stop_request}")
        try:
            await self.instance_async_client.StopWorkspaceInstance(stop_request)
        except Exception:
            raise
        return workspace_instance_id

    @property
    def instance_async_client(self) -> WorkspaceInstanceServiceStub:
        try:
            return self._instance_async_client
        except AttributeError:
            self._instance_async_client = WorkspaceInstanceServiceStub(self.async_channel)
            return self._instance_async_client

    @property
    def definition_async_client(self) -> WorkspaceRegistryServiceStub:
        try:
            return self._definition_async_client
        except AttributeError:
            self._definition_async_client = WorkspaceRegistryServiceStub(self.async_channel)
            return self._definition_async_client

    @property
    def sync_channel(self) -> grpc.Channel:
        try:
            return self._sync_channel
        except AttributeError:
            self._sync_channel, self._org = _get_channel_with_org(self.config.platform)
            return self._sync_channel

    @property
    def async_channel(self) -> grpc.aio.Channel:
        from union.filesystems._endpoint import _create_secure_channel_from_config

        try:
            return self._async_channel
        except AttributeError:
            self._async_channel = _create_secure_channel_from_config(self.config.platform, self.sync_channel)
            return self._async_channel

    @property
    def org(self) -> Optional[str]:
        try:
            return self._org
        except AttributeError:
            self._sync_channel, self._org = _get_channel_with_org(self.config.platform)
            if self._org is None or self._org == "":
                self._org = self.config.platform.endpoint.split(".")[0]
            return self._org


def _generate_random_str(N: int):
    items = string.ascii_letters + string.digits
    return "".join(random.choice(items) for _ in range(N))


def _generate_workspace_name(name: str):
    """Generate workspace name prefixed by name."""
    return f"{name}-{_generate_random_str(VERSION_STRING_LENGTH)}"


def _get_workspace_name(version):
    return version[: -(VERSION_STRING_LENGTH + 1)]


def _get_workspace_executions(remote: UnionRemote, project: str, domain: str):
    executions, _ = remote.client.list_executions_paginated(
        project=project,
        domain=domain,
        limit=1000,
        filters=[
            Equal("task.name", "union.workspace._vscode.workspace"),
            ValueIn("phase", ["RUNNING", "QUEUED"]),
        ],
    )
    return [(e, _get_workspace_name(e.spec.launch_plan.version)) for e in executions]


def _get_vscode_link(remote: UnionRemote, execution: Execution) -> str:
    if execution.closure.phase != WorkflowExecutionPhase.RUNNING:
        return "Unavailable"

    workflow_execution = remote.fetch_execution(
        project=execution.id.project,
        domain=execution.id.domain,
        name=execution.id.name,
    )
    workflow_execution = remote.sync_execution(workflow_execution, sync_nodes=True)

    if not workflow_execution.node_executions:
        return "Unavailable"

    all_keys = [e for e in workflow_execution.node_executions if e != "start-node"]
    if not all_keys:
        return "Unavailable"

    vscode_key = all_keys[0]
    vscode_node = workflow_execution.node_executions[vscode_key]

    if vscode_node.closure.phase != TaskExecutionPhase.RUNNING or not vscode_node.task_executions:
        return "Unavailable"

    task_execution = vscode_node.task_executions[-1]
    if task_execution.closure.phase != TaskExecutionPhase.RUNNING:
        return "Unavailable"

    for log in task_execution.closure.logs:
        if VSCODE_DEBUGGER_LOG_NAME in log.name:
            return remote.generate_console_http_domain() + log.uri



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: authorizer/definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import deployment_pb2 as common_dot_deployment__pb2
from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.common import identity_pb2 as common_dot_identity__pb2
from union.internal.common import policy_pb2 as common_dot_policy__pb2
from union.internal.common import role_pb2 as common_dot_role__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61uthorizer/definition.proto\x12\x13\x63loudidl.authorizer\x1a\x17\x63ommon/deployment.proto\x1a\x17\x63ommon/identifier.proto\x1a\x15\x63ommon/identity.proto\x1a\x13\x63ommon/policy.proto\x1a\x11\x63ommon/role.proto\"\x85\x01\n\x0eRoleAssignment\x12\x38\n\x07role_id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.RoleIdentifierR\x06roleId\x12\x39\n\nidentities\x18\x02 \x03(\x0b\x32\x19.cloudidl.common.IdentityR\nidentities\"\x8d\x01\n\x10PolicyAssignment\x12>\n\tpolicy_id\x18\x01 \x01(\x0b\x32!.cloudidl.common.PolicyIdentifierR\x08policyId\x12\x39\n\nidentities\x18\x02 \x03(\x0b\x32\x19.cloudidl.common.IdentityR\nidentities\"\xb1\x01\n\x12IdentityAssignment\x12\x35\n\x08identity\x18\x01 \x01(\x0b\x32\x19.cloudidl.common.IdentityR\x08identity\x12/\n\x05roles\x18\x02 \x03(\x0b\x32\x15.cloudidl.common.RoleB\x02\x18\x01R\x05roles\x12\x33\n\x08policies\x18\x03 \x03(\x0b\x32\x17.cloudidl.common.PolicyR\x08policies\"\xc1\x01\n\x13OrganizationOptions\x12U\n\x12\x61uthorization_type\x18\x01 \x01(\x0e\x32&.cloudidl.authorizer.AuthorizationTypeR\x11\x61uthorizationType\x12\x16\n\x06parent\x18\x02 \x01(\tR\x06parent\x12;\n\ndeployment\x18\x03 \x01(\x0e\x32\x1b.cloudidl.common.DeploymentR\ndeployment*T\n\x11\x41uthorizationType\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14TYPE_DEPRECATED_ROLE\x10\x01\x12\x0f\n\x0bTYPE_POLICY\x10\x02\x42\xc8\x01\n\x17\x63om.cloudidl.authorizerB\x0f\x44\x65\x66initionProtoH\x02P\x01Z-github.com/unionai/cloud/gen/pb-go/authorizer\xa2\x02\x03\x43\x41X\xaa\x02\x13\x43loudidl.Authorizer\xca\x02\x13\x43loudidl\\Authorizer\xe2\x02\x1f\x43loudidl\\Authorizer\\GPBMetadata\xea\x02\x14\x43loudidl::Authorizerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'authorizer.definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.cloudidl.authorizerB\017DefinitionProtoH\002P\001Z-github.com/unionai/cloud/gen/pb-go/authorizer\242\002\003CAX\252\002\023Cloudidl.Authorizer\312\002\023Cloudidl\\Authorizer\342\002\037Cloudidl\\Authorizer\\GPBMetadata\352\002\024Cloudidl::Authorizer'
  _IDENTITYASSIGNMENT.fields_by_name['roles']._options = None
  _IDENTITYASSIGNMENT.fields_by_name['roles']._serialized_options = b'\030\001'
  _globals['_AUTHORIZATIONTYPE']._serialized_start=821
  _globals['_AUTHORIZATIONTYPE']._serialized_end=905
  _globals['_ROLEASSIGNMENT']._serialized_start=166
  _globals['_ROLEASSIGNMENT']._serialized_end=299
  _globals['_POLICYASSIGNMENT']._serialized_start=302
  _globals['_POLICYASSIGNMENT']._serialized_end=443
  _globals['_IDENTITYASSIGNMENT']._serialized_start=446
  _globals['_IDENTITYASSIGNMENT']._serialized_end=623
  _globals['_ORGANIZATIONOPTIONS']._serialized_start=626
  _globals['_ORGANIZATIONOPTIONS']._serialized_end=819
# @@protoc_insertion_point(module_scope)



import os

REMOTE_MODEL_PATH = os.getenv("UNION_MODEL_LOADER_REMOTE_MODEL_PATH")
LOCAL_MODEL_PATH = os.getenv("UNION_MODEL_LOADER_LOCAL_MODEL_PATH", "/srv/model")
CHUNK_SIZE = int(os.getenv("UNION_MODEL_LOADER_CHUNK_SIZE", str(16 * 1024 * 1024)))
MAX_CONCURRENCY = int(os.getenv("UNION_MODEL_LOADER_MAX_CONCURRENCY", str(32)))
STREAM_SAFETENSORS = os.getenv("UNION_MODEL_LOADER_STREAM_SAFETENSORS", "false").lower() == "true"



from google.api import annotations_pb2 as _annotations_pb2
from union.internal.workspace import workspace_definition_payload_pb2 as _workspace_definition_payload_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



import asyncio
import logging
import traceback
from typing import Optional

import aiofiles
import fsspec
import grpc
from fsspec.asyn import AbstractAsyncStreamedFile, AsyncFileSystem, get_loop, sync
from fsspec.spec import AbstractBufferedFile

from union._async import patch_exception_handler
from union.filesystems._async_utils import create_channel_with_loop
from union.filesystems._endpoint import _create_channel
from union.internal.common import list_pb2
from union.internal.objectstore.definition_pb2 import Key, Metadata, Object
from union.internal.objectstore.metadata_service_pb2_grpc import MetadataStoreServiceStub
from union.internal.objectstore.payload_pb2 import (
    DeleteRequest,
    GetRequest,
    GetResponse,
    HeadRequest,
    HeadResponse,
    ListRequest,
    ListResponse,
    MetadataResponse,
    PutRequest,
)

_DEFAULT_LOGGER = logging.getLogger(__name__)


_FS_PREFIX = "unionmeta://"

# Shorter default protocol for unionmeta
_FS_PREFIX_SHORT = "ums://"


def _prefix_path(path: str) -> str:
    if path.startswith(_FS_PREFIX):
        return path

    if path.startswith(_FS_PREFIX_SHORT):
        return path

    return f"{_FS_PREFIX}{path.rstrip('/')}"


class AsyncUnionMetaFS(AsyncFileSystem):
    mirror_sync_methods = True
    cachable = False

    def __init__(
        self,
        logger: logging.Logger = _DEFAULT_LOGGER,
        loop: Optional[asyncio.AbstractEventLoop] = None,
        *args,
        **kwargs,
    ):
        if loop is None:
            loop = get_loop()
            remote_connection = create_channel_with_loop(loop, _create_channel)
        else:
            if not loop.is_running():
                raise ValueError("If a loop is passed in, it must be running")
            remote_connection = _create_channel()
        patch_exception_handler(loop)

        self._client = MetadataStoreServiceStub(remote_connection.channel)

        super().__init__(loop=loop, *args, **kwargs)
        self._metadata = None
        self.blocksize = 2**18  # 2MB
        self._logger = logger

    def _prefix_path(self, path):
        if path.startswith(f"{self.protocol}://"):
            return path
        return f"{self.protocol}://{path}"

    async def _ensure_metadata(self) -> MetadataResponse:
        if self._metadata is not None:
            return self._metadata

        self._metadata = await self._client.Metadata(Metadata())
        return self._metadata

    async def _rm_file(self, path, **kwargs):
        path = self._prefix_path(path)
        self._logger.info(f"Deleting {path}")
        await self._client.Delete(DeleteRequest(key=Key(key=path)))

    async def _cp_file(self, path1, path2, **kwargs):
        # TODO: Implement cp in remote metadata store
        _, temp_filename = await aiofiles.tempfile.TemporaryFile(suffix=".yaml")
        await self._get_file(path1, temp_filename, **kwargs)
        await self._put_file(temp_filename, path2, **kwargs)

    async def _pipe_file(self, path, value, **kwargs):
        path = self._prefix_path(path)
        req = PutRequest(metadata=Metadata(), key=Key(key=path), object=Object(contents=value))
        try:
            await self._client.Put(req)
        except Exception as e:
            traceback.print_exception(e)
            raise

    async def _cat_file(self, path, start=None, end=None, **kwargs):
        path = self._prefix_path(path)
        req = GetRequest(key=Key(key=path))
        try:
            res: GetResponse = await self._client.Get(req)
            return res.object.contents
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                raise FileNotFoundError(path)
            traceback.print_exception(e)
            raise e
        except Exception as e:
            traceback.print_exception(e)
            raise

    async def _put_file(self, lpath, rpath, **kwargs):
        rpath = self._prefix_path(rpath)
        async with aiofiles.open(lpath, "rb") as f:
            await self._pipe_file(rpath, await f.read(), **kwargs)

    async def _get_file(self, rpath, lpath, **kwargs):
        rpath = self._prefix_path(rpath)
        async with aiofiles.open(lpath, "wb") as f:
            await f.write(await self._cat_file(rpath, **kwargs))

    async def _info(self, path, **kwargs):
        self._logger.info(f"Info: {path}")
        path = self._prefix_path(path)
        req = HeadRequest(key=Key(key=path))
        try:
            res: HeadResponse = await self._client.Head(req)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                # If there is no key at that location, check if it's a prefix
                nested_keys = await self._ls(path=path, detail=False)
                if len(nested_keys) > 0:
                    return {
                        "name": path,
                        "type": "directory",
                        "size": 0,
                        "StorageClass": "DIRECTORY",
                    }
                raise FileNotFoundError(path)
            traceback.print_exception(e)
            raise
        except Exception as e:
            traceback.print_exception(e)
            raise

        return {
            "size": res.size_bytes,
            "etag": res.etag,
            "tags": res.metadata.tag if res.metadata and res.metadata.tag else {},
            "type": "file",
        }

    async def _ls(self, path, detail=True, **kwargs):
        path = self._prefix_path(path)
        if not path.endswith("*"):
            path = f"{path}*"

        req = ListRequest(
            request=list_pb2.ListRequest(
                limit=1000,
                filters=[
                    list_pb2.Filter(
                        field="prefix",
                        function=list_pb2.Filter.GREATER_THAN_OR_EQUAL,
                        values=[path],
                    )
                ],
            )
        )

        items = []
        while True:
            res: ListResponse = await self._client.List(req)
            for k in res.keys:
                items.append(
                    {
                        "name": k.key,
                        "type": "file",
                        # The API doesn't return information about the object sizes for now.
                        "size": 0,
                    }
                )
            for d in res.directories:
                items.append(
                    {
                        "name": d.key,
                        "type": "directory",
                        # The API doesn't return information about the object sizes for now.
                        "size": 0,
                    }
                )
            if not res.next_token or res.next_token == "":
                break

        return items

    async def open_async(self, path: str, mode="rb", **kwargs) -> AbstractAsyncStreamedFile:
        if "b" not in mode or kwargs.get("compression"):
            raise ValueError
        path = self._prefix_path(path)
        self._logger.debug(f"Opening file async: {path}")
        m = await self._ensure_metadata()
        return AsyncUnionStreamFile(self, self._client, path, mode, block_size=m.max_part_size_bytes, **kwargs)

    def _open(
        self,
        path,
        mode="rb",
        block_size=None,
        autocommit=True,
        cache_options=None,
        **kwargs,
    ):
        if "b" not in mode or kwargs.get("compression"):
            raise ValueError
        path = _prefix_path(path)
        self._logger.debug(f"Opening file: {path}")
        m = sync(self.loop, self._ensure_metadata)
        return UnionStreamFile(
            self._client,
            fs=self,
            path=path,
            mode=mode,
            block_size=m.max_part_size_bytes,
            autocommit=autocommit,
            cache_options=cache_options,
            **kwargs,
        )


class AsyncUnionStreamFile(AbstractAsyncStreamedFile):
    """
    AsyncUnionStreamFile implements the AbstractAsyncStreamedFile interface for the UnionMetaFS.
    It can be used to read and write files using async.io apis.
    """

    def __init__(
        self,
        fs: AsyncUnionMetaFS,
        object_store_client: MetadataStoreServiceStub,
        path: str,
        mode: str,
        **kwargs,
    ):
        super().__init__(fs, path, mode, **kwargs)
        self._client = object_store_client
        self._part_number = 1
        self._bytes = b""

    async def _fetch_range(self, start: int, end: int) -> bytes:
        if not self._bytes:
            get_response = await self._client.Get(GetRequest(key=Key(key=_prefix_path(self.path))))
            self._bytes = get_response.object.contents

        return self._bytes[start:end]

    async def _initiate_upload(self):
        if self.autocommit and self.tell() < self.blocksize:
            # only happens when closing small file, use on-shot PUT
            return

        raise ValueError("The file is too big to be uploaded to union meta store. Please use a smaller file.")

    async def _upload_chunk(self, final: bool = False):
        if self.autocommit and final and self.tell() < self.blocksize:
            # only happens when closing small file, use on-shot PUT
            self.buffer.seek(0, 0)
            await self._client.Put(
                PutRequest(key=Key(key=_prefix_path(self.path)), object=Object(contents=self.buffer.getvalue()))
            )
            return True


class UnionStreamFile(AbstractBufferedFile):
    def __init__(
        self,
        client: MetadataStoreServiceStub,
        fs: AsyncUnionMetaFS,
        path,
        mode="rb",
        block_size="default",
        autocommit=True,
        cache_type="readahead",
        cache_options=None,
        size=None,
        **kwargs,
    ):
        super().__init__(
            fs,
            path,
            mode,
            block_size,
            autocommit,
            cache_type,
            cache_options,
            size,
            **kwargs,
        )
        self._client = client
        self._bytes = b""

    def _upload_chunk(self, final=False):
        if self.autocommit and final and self.tell() < self.blocksize:
            # only happens when closing small file, use on-shot PUT
            self.buffer.seek(0, 0)
            sync(
                self.fs.loop,
                self._client.Put,
                PutRequest(key=Key(key=_prefix_path(self.path)), object=Object(contents=self.buffer.getvalue())),
            )
            return True

    def _initiate_upload(self):
        if self.autocommit and self.tell() < self.blocksize:
            # only happens when closing small file, use on-shot PUT
            return

        raise ValueError("The file is too big to be uploaded to union meta store. Please use a smaller file.")

    def _fetch_range(self, start, end):
        if not self._bytes:
            get_response = sync(self.fs.loop, self._client.Get, GetRequest(key=Key(key=_prefix_path(self.path))))
            self._bytes = get_response.object.contents

        return self._bytes[start:end]


fsspec.register_implementation(name="unionmeta", cls=AsyncUnionMetaFS, clobber=True)

# Shorter default protocol for unionmeta
fsspec.register_implementation(name="ums", cls=AsyncUnionMetaFS, clobber=True)



import importlib.util
import inspect
import os
import sys
from pathlib import Path
from textwrap import dedent
from typing import Dict

import rich_click as click

from union._config import _DEFAULT_DOMAIN, _DEFAULT_PROJECT_BYOC, _GetDefaultProject
from union.app import App


class ApplicationCommand(click.RichCommand):
    def __init__(
        self,
        app: App,
        project: str,
        domain: str,
        debug: bool,
        *args,
        **kwargs,
    ):
        self.app = app
        self.project = project
        self.domain = domain
        self.debug = debug

        kwargs["params"] = [
            click.Option(
                param_decls=[f"--{app_input.name}"],
                required=False,
                type=str,
                help=f"Set {app_input.name}",
            )
            for app_input in app.inputs
        ]

        super().__init__(*args, **kwargs)

    def invoke(self, ctx):
        app = self.app

        # Check if there are any dynamic inputs
        for user_input in app.inputs:
            key = user_input.name
            if key is not None and ctx.params.get(key):
                user_input.value = ctx.params[key]

        if self.debug:
            app.command = [
                "sh",
                "-c",
                f"set -e; echo '🚀 Setting up vscode 🛠️'; curl -fsSL https://code-server.dev/install.sh | sh -s --"
                f" --method standalone; "
                f"~/.local/bin/code-server --bind-addr 0.0.0.0:{app.port} --disable-workspace-trust --auth none;",
            ]

            app.args = []
            app.min_replicas = 1
            app.max_replicas = 1

        from union.remote import UnionRemote

        remote = UnionRemote(default_project=self.project, default_domain=self.domain)
        remote.deploy_app(app)


class ApplicationForFileGroup(click.RichGroup):
    def __init__(
        self,
        filename: Path,
        project: str,
        domain: str,
        debug: bool,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        if not filename.exists():
            raise click.ClickException(f"{filename} does not exists")
        self.filename = filename
        self.project = project
        self.domain = domain
        self.debug = debug
        # self.commands = self._get_commands()

    @property
    def apps(self) -> Dict[str, App]:
        try:
            return self._apps
        except AttributeError:
            cwd = Path.cwd()
            sys.path.append(os.fspath(cwd))

            module_name = self.filename.stem
            module_path = self.filename.absolute().parent

            spec = importlib.util.spec_from_file_location(module_name, self.filename)
            module = importlib.util.module_from_spec(spec)
            sys.modules[module_name] = module
            spec.loader.exec_module(module)
            sys.path.append(os.fspath(module_path))

            from union.app._models import APP_REGISTRY

            app_name_to_module = {obj.name: module for _, obj in inspect.getmembers(module) if isinstance(obj, App)}

            filename_relative_to_cwd = self.filename.absolute().relative_to(cwd)
            module_name_relative_to_cwd = ".".join(filename_relative_to_cwd.with_suffix("").parts)

            self._apps = {
                k: v._attach_registration_scope(
                    app_name_to_module.get(k), module_name_relative_to_cwd
                )._resolve_include(module_path, cwd)
                for k, v in APP_REGISTRY.apps.items()
            }
            return self._apps

    def _get_commands(self, ctx: click.Context):
        commands = {}
        apps = self.apps
        for app_name in apps.keys():
            commands[app_name] = ApplicationCommand(
                app=apps[app_name],
                name=app_name,
                project=ctx.params.get("project", _DEFAULT_PROJECT_BYOC),
                domain=ctx.params.get("domain", _DEFAULT_DOMAIN),
                debug=ctx.params.get("debug", False),
            )
        return commands

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        self.commands = self._get_commands(ctx)
        super().format_help(ctx, formatter)

    def list_commands(self, ctx):
        return list(self.apps.keys())

    def get_command(self, ctx, app_name):
        try:
            return ApplicationCommand(
                app=self.apps[app_name],
                name=app_name,
                project=self.project,
                domain=self.domain,
                debug=self.debug,
            )
        except KeyError:
            valid_names = list(self.apps.keys())
            err_msg = dedent(f"""\
                '{app_name}' is not a valid app name in {self.filename}

                Valid names are: {valid_names}""")
            raise click.ClickException(err_msg)


class DeployApplicationGroupForFiles(click.RichGroup):
    def __init__(self, command_name: str, *args, **kwargs):
        kwargs["params"] = [
            click.Option(
                param_decls=["-p", "--project"],
                required=False,
                type=str,
                default=_GetDefaultProject(previous_default=_DEFAULT_PROJECT_BYOC),
                help=f"Project to run {command_name}",
                show_default=True,
            ),
            click.Option(
                param_decls=["-d", "--domain"],
                required=False,
                type=str,
                default=_DEFAULT_DOMAIN,
                help=f"Domain to run {command_name}",
                show_default=True,
            ),
            click.Option(
                param_decls=["-n", "--name"],
                required=False,
                type=str,
                help="Application name to start",
                show_default=True,
            ),
            click.Option(
                param_decls=["--debug"],
                required=False,
                type=bool,
                is_flag=True,
                default=False,
                help="Start app in debug mode (vscode server). It requires `sh` and `curl` to be available on the"
                " image. VSCode Server will be downloaded on demand.",
                show_default=True,
            ),
        ]
        super().__init__(*args, **kwargs)
        self.command_name = command_name

    @property
    def files(self):
        try:
            return self._files
        except AttributeError:
            self._files = [os.fspath(p) for p in Path(".").glob("*.py") if p.name != "__init__.py"]
            return self._files

    def _get_commands(self, ctx: click.Context):
        commands = {}
        for filename in self.files:
            commands[filename] = ApplicationForFileGroup(
                filename=Path(filename),
                name=filename,
                help=f"{self.command_name} application in {filename}",
                project=ctx.params.get("project", _DEFAULT_PROJECT_BYOC),
                domain=ctx.params.get("domain", _DEFAULT_DOMAIN),
                debug=ctx.params.get("debug", False),
            )
        return commands

    def invoke(self, ctx):
        if "name" in ctx.params and not ctx.protected_args:
            # Command is invoked with just `--name`
            project = ctx.params.get("project", _DEFAULT_PROJECT_BYOC)
            domain = ctx.params.get("domain", _DEFAULT_DOMAIN)
            name = ctx.params.get("name")

            from union.remote import UnionRemote

            remote = UnionRemote(default_project=project, default_domain=domain)
            remote._app_remote.start(name)
            return

        return super().invoke(ctx)

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        self.commands = self._get_commands(ctx)
        super().format_help(ctx, formatter)

    def list_commands(self, ctx):
        return self.files

    def get_command(self, ctx: click.Context, filename: str) -> click.Command:
        return ApplicationForFileGroup(
            filename=Path(filename),
            name=filename,
            project=ctx.params.get("project", _DEFAULT_PROJECT_BYOC),
            domain=ctx.params.get("domain", _DEFAULT_DOMAIN),
            debug=ctx.params.get("debug", False),
            help=f"{self.command_name} application in {filename}",
        )



import sys
from typing import Any


def _is_fastapi_app(app: Any) -> bool:
    """
    Return True if app is a FastAPI app.
    """
    try:
        fastapi = sys.modules["fastapi"]
    except KeyError:
        return False
    return isinstance(app, fastapi.FastAPI)



from union.internal.common import deployment_pb2 as _deployment_pb2
from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import identity_pb2 as _identity_pb2
from union.internal.common import policy_pb2 as _policy_pb2
from union.internal.common import role_pb2 as _role_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AuthorizationType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    TYPE_UNSPECIFIED: _ClassVar[AuthorizationType]
    TYPE_DEPRECATED_ROLE: _ClassVar[AuthorizationType]
    TYPE_POLICY: _ClassVar[AuthorizationType]
TYPE_UNSPECIFIED: AuthorizationType
TYPE_DEPRECATED_ROLE: AuthorizationType
TYPE_POLICY: AuthorizationType

class RoleAssignment(_message.Message):
    __slots__ = ["role_id", "identities"]
    ROLE_ID_FIELD_NUMBER: _ClassVar[int]
    IDENTITIES_FIELD_NUMBER: _ClassVar[int]
    role_id: _identifier_pb2.RoleIdentifier
    identities: _containers.RepeatedCompositeFieldContainer[_identity_pb2.Identity]
    def __init__(self, role_id: _Optional[_Union[_identifier_pb2.RoleIdentifier, _Mapping]] = ..., identities: _Optional[_Iterable[_Union[_identity_pb2.Identity, _Mapping]]] = ...) -> None: ...

class PolicyAssignment(_message.Message):
    __slots__ = ["policy_id", "identities"]
    POLICY_ID_FIELD_NUMBER: _ClassVar[int]
    IDENTITIES_FIELD_NUMBER: _ClassVar[int]
    policy_id: _identifier_pb2.PolicyIdentifier
    identities: _containers.RepeatedCompositeFieldContainer[_identity_pb2.Identity]
    def __init__(self, policy_id: _Optional[_Union[_identifier_pb2.PolicyIdentifier, _Mapping]] = ..., identities: _Optional[_Iterable[_Union[_identity_pb2.Identity, _Mapping]]] = ...) -> None: ...

class IdentityAssignment(_message.Message):
    __slots__ = ["identity", "roles", "policies"]
    IDENTITY_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    POLICIES_FIELD_NUMBER: _ClassVar[int]
    identity: _identity_pb2.Identity
    roles: _containers.RepeatedCompositeFieldContainer[_role_pb2.Role]
    policies: _containers.RepeatedCompositeFieldContainer[_policy_pb2.Policy]
    def __init__(self, identity: _Optional[_Union[_identity_pb2.Identity, _Mapping]] = ..., roles: _Optional[_Iterable[_Union[_role_pb2.Role, _Mapping]]] = ..., policies: _Optional[_Iterable[_Union[_policy_pb2.Policy, _Mapping]]] = ...) -> None: ...

class OrganizationOptions(_message.Message):
    __slots__ = ["authorization_type", "parent", "deployment"]
    AUTHORIZATION_TYPE_FIELD_NUMBER: _ClassVar[int]
    PARENT_FIELD_NUMBER: _ClassVar[int]
    DEPLOYMENT_FIELD_NUMBER: _ClassVar[int]
    authorization_type: AuthorizationType
    parent: str
    deployment: _deployment_pb2.Deployment
    def __init__(self, authorization_type: _Optional[_Union[AuthorizationType, str]] = ..., parent: _Optional[str] = ..., deployment: _Optional[_Union[_deployment_pb2.Deployment, str]] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from union.internal.authorizer import payload_pb2 as authorizer_dot_payload__pb2


class AuthorizerServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Authorize = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/Authorize',
                request_serializer=authorizer_dot_payload__pb2.AuthorizeRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.AuthorizeResponse.FromString,
                )
        self.CreateIdentity = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreateIdentity',
                request_serializer=authorizer_dot_payload__pb2.CreateIdentityRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreateIdentityResponse.FromString,
                )
        self.DeleteIdentity = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeleteIdentity',
                request_serializer=authorizer_dot_payload__pb2.DeleteIdentityRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeleteIdentityResponse.FromString,
                )
        self.AssignIdentity = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/AssignIdentity',
                request_serializer=authorizer_dot_payload__pb2.AssignIdentityRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.AssignIdentityResponse.FromString,
                )
        self.UnassignIdentity = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/UnassignIdentity',
                request_serializer=authorizer_dot_payload__pb2.UnassignIdentityRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.UnassignIdentityResponse.FromString,
                )
        self.GetIdentityAssignments = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/GetIdentityAssignments',
                request_serializer=authorizer_dot_payload__pb2.GetIdentityAssignmentRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.GetIdentityAssignmentResponse.FromString,
                )
        self.ListIdentityAssignments = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/ListIdentityAssignments',
                request_serializer=authorizer_dot_payload__pb2.ListIdentityAssignmentsRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.ListIdentityAssignmentsResponse.FromString,
                )
        self.GetPolicyAssignment = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/GetPolicyAssignment',
                request_serializer=authorizer_dot_payload__pb2.GetPolicyAssignmentRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.GetPolicyAssignmentResponse.FromString,
                )
        self.CreateResource = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreateResource',
                request_serializer=authorizer_dot_payload__pb2.CreateResourceRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreateResourceResponse.FromString,
                )
        self.DeleteResource = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeleteResource',
                request_serializer=authorizer_dot_payload__pb2.DeleteResourceRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeleteResourceResponse.FromString,
                )
        self.CreateOrganization = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreateOrganization',
                request_serializer=authorizer_dot_payload__pb2.CreateOrganizationRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreateOrganizationResponse.FromString,
                )
        self.GetOrganization = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/GetOrganization',
                request_serializer=authorizer_dot_payload__pb2.GetOrganizationRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.GetOrganizationResponse.FromString,
                )
        self.DeleteOrganization = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeleteOrganization',
                request_serializer=authorizer_dot_payload__pb2.DeleteOrganizationRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeleteOrganizationResponse.FromString,
                )
        self.CreatePolicy = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreatePolicy',
                request_serializer=authorizer_dot_payload__pb2.CreatePolicyRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreatePolicyResponse.FromString,
                )
        self.GetPolicy = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/GetPolicy',
                request_serializer=authorizer_dot_payload__pb2.GetPolicyRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.GetPolicyResponse.FromString,
                )
        self.DeletePolicy = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeletePolicy',
                request_serializer=authorizer_dot_payload__pb2.DeletePolicyRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeletePolicyResponse.FromString,
                )
        self.ListPolicies = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/ListPolicies',
                request_serializer=authorizer_dot_payload__pb2.ListPoliciesRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.ListPoliciesResponse.FromString,
                )
        self.CreatePolicyBinding = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreatePolicyBinding',
                request_serializer=authorizer_dot_payload__pb2.CreatePolicyBindingRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreatePolicyBindingResponse.FromString,
                )
        self.DeletePolicyBinding = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeletePolicyBinding',
                request_serializer=authorizer_dot_payload__pb2.DeletePolicyBindingRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeletePolicyBindingResponse.FromString,
                )
        self.CreateRole = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreateRole',
                request_serializer=authorizer_dot_payload__pb2.CreateRoleRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreateRoleResponse.FromString,
                )
        self.GetRole = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/GetRole',
                request_serializer=authorizer_dot_payload__pb2.GetRoleRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.GetRoleResponse.FromString,
                )
        self.DeleteRole = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeleteRole',
                request_serializer=authorizer_dot_payload__pb2.DeleteRoleRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeleteRoleResponse.FromString,
                )
        self.ListRoles = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/ListRoles',
                request_serializer=authorizer_dot_payload__pb2.ListRolesRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.ListRolesResponse.FromString,
                )
        self.CreateRoleAction = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/CreateRoleAction',
                request_serializer=authorizer_dot_payload__pb2.CreateRoleActionRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.CreateRoleActionResponse.FromString,
                )
        self.DeleteRoleAction = channel.unary_unary(
                '/cloudidl.authorizer.AuthorizerService/DeleteRoleAction',
                request_serializer=authorizer_dot_payload__pb2.DeleteRoleActionRequest.SerializeToString,
                response_deserializer=authorizer_dot_payload__pb2.DeleteRoleActionResponse.FromString,
                )


class AuthorizerServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Authorize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateIdentity(self, request, context):
        """Creates an identity in the authorizer store
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteIdentity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AssignIdentity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnassignIdentity(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIdentityAssignments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListIdentityAssignments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPolicyAssignment(self, request, context):
        """Returns the identities assigned to a policy.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateResource(self, request, context):
        """Creates a resource if it does not already exist in the authorizer store. Returns no errors when a resource already exists.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteResource(self, request, context):
        """Deletes a resource if it already exists in the authorizer store. Returns no errors when a resource doesn't exists.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateOrganization(self, request, context):
        """Creates an organization in the authorization store. Returns no errors when an organization already exists.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOrganization(self, request, context):
        """Get an organization in the authorization store.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteOrganization(self, request, context):
        """Deletes an organization in the authorization store. Returns no errors when an organization already doesn't exist.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePolicy(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPolicy(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeletePolicy(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPolicies(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePolicyBinding(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeletePolicyBinding(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateRole(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRole(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteRole(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListRoles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateRoleAction(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteRoleAction(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AuthorizerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Authorize': grpc.unary_unary_rpc_method_handler(
                    servicer.Authorize,
                    request_deserializer=authorizer_dot_payload__pb2.AuthorizeRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.AuthorizeResponse.SerializeToString,
            ),
            'CreateIdentity': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateIdentity,
                    request_deserializer=authorizer_dot_payload__pb2.CreateIdentityRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreateIdentityResponse.SerializeToString,
            ),
            'DeleteIdentity': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteIdentity,
                    request_deserializer=authorizer_dot_payload__pb2.DeleteIdentityRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeleteIdentityResponse.SerializeToString,
            ),
            'AssignIdentity': grpc.unary_unary_rpc_method_handler(
                    servicer.AssignIdentity,
                    request_deserializer=authorizer_dot_payload__pb2.AssignIdentityRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.AssignIdentityResponse.SerializeToString,
            ),
            'UnassignIdentity': grpc.unary_unary_rpc_method_handler(
                    servicer.UnassignIdentity,
                    request_deserializer=authorizer_dot_payload__pb2.UnassignIdentityRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.UnassignIdentityResponse.SerializeToString,
            ),
            'GetIdentityAssignments': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIdentityAssignments,
                    request_deserializer=authorizer_dot_payload__pb2.GetIdentityAssignmentRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.GetIdentityAssignmentResponse.SerializeToString,
            ),
            'ListIdentityAssignments': grpc.unary_unary_rpc_method_handler(
                    servicer.ListIdentityAssignments,
                    request_deserializer=authorizer_dot_payload__pb2.ListIdentityAssignmentsRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.ListIdentityAssignmentsResponse.SerializeToString,
            ),
            'GetPolicyAssignment': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPolicyAssignment,
                    request_deserializer=authorizer_dot_payload__pb2.GetPolicyAssignmentRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.GetPolicyAssignmentResponse.SerializeToString,
            ),
            'CreateResource': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateResource,
                    request_deserializer=authorizer_dot_payload__pb2.CreateResourceRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreateResourceResponse.SerializeToString,
            ),
            'DeleteResource': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteResource,
                    request_deserializer=authorizer_dot_payload__pb2.DeleteResourceRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeleteResourceResponse.SerializeToString,
            ),
            'CreateOrganization': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateOrganization,
                    request_deserializer=authorizer_dot_payload__pb2.CreateOrganizationRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreateOrganizationResponse.SerializeToString,
            ),
            'GetOrganization': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOrganization,
                    request_deserializer=authorizer_dot_payload__pb2.GetOrganizationRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.GetOrganizationResponse.SerializeToString,
            ),
            'DeleteOrganization': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteOrganization,
                    request_deserializer=authorizer_dot_payload__pb2.DeleteOrganizationRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeleteOrganizationResponse.SerializeToString,
            ),
            'CreatePolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePolicy,
                    request_deserializer=authorizer_dot_payload__pb2.CreatePolicyRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreatePolicyResponse.SerializeToString,
            ),
            'GetPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPolicy,
                    request_deserializer=authorizer_dot_payload__pb2.GetPolicyRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.GetPolicyResponse.SerializeToString,
            ),
            'DeletePolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.DeletePolicy,
                    request_deserializer=authorizer_dot_payload__pb2.DeletePolicyRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeletePolicyResponse.SerializeToString,
            ),
            'ListPolicies': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPolicies,
                    request_deserializer=authorizer_dot_payload__pb2.ListPoliciesRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.ListPoliciesResponse.SerializeToString,
            ),
            'CreatePolicyBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePolicyBinding,
                    request_deserializer=authorizer_dot_payload__pb2.CreatePolicyBindingRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreatePolicyBindingResponse.SerializeToString,
            ),
            'DeletePolicyBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.DeletePolicyBinding,
                    request_deserializer=authorizer_dot_payload__pb2.DeletePolicyBindingRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeletePolicyBindingResponse.SerializeToString,
            ),
            'CreateRole': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateRole,
                    request_deserializer=authorizer_dot_payload__pb2.CreateRoleRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreateRoleResponse.SerializeToString,
            ),
            'GetRole': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRole,
                    request_deserializer=authorizer_dot_payload__pb2.GetRoleRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.GetRoleResponse.SerializeToString,
            ),
            'DeleteRole': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteRole,
                    request_deserializer=authorizer_dot_payload__pb2.DeleteRoleRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeleteRoleResponse.SerializeToString,
            ),
            'ListRoles': grpc.unary_unary_rpc_method_handler(
                    servicer.ListRoles,
                    request_deserializer=authorizer_dot_payload__pb2.ListRolesRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.ListRolesResponse.SerializeToString,
            ),
            'CreateRoleAction': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateRoleAction,
                    request_deserializer=authorizer_dot_payload__pb2.CreateRoleActionRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.CreateRoleActionResponse.SerializeToString,
            ),
            'DeleteRoleAction': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteRoleAction,
                    request_deserializer=authorizer_dot_payload__pb2.DeleteRoleActionRequest.FromString,
                    response_serializer=authorizer_dot_payload__pb2.DeleteRoleActionResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cloudidl.authorizer.AuthorizerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AuthorizerService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Authorize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/Authorize',
            authorizer_dot_payload__pb2.AuthorizeRequest.SerializeToString,
            authorizer_dot_payload__pb2.AuthorizeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateIdentity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreateIdentity',
            authorizer_dot_payload__pb2.CreateIdentityRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreateIdentityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteIdentity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeleteIdentity',
            authorizer_dot_payload__pb2.DeleteIdentityRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeleteIdentityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AssignIdentity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/AssignIdentity',
            authorizer_dot_payload__pb2.AssignIdentityRequest.SerializeToString,
            authorizer_dot_payload__pb2.AssignIdentityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnassignIdentity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/UnassignIdentity',
            authorizer_dot_payload__pb2.UnassignIdentityRequest.SerializeToString,
            authorizer_dot_payload__pb2.UnassignIdentityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIdentityAssignments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/GetIdentityAssignments',
            authorizer_dot_payload__pb2.GetIdentityAssignmentRequest.SerializeToString,
            authorizer_dot_payload__pb2.GetIdentityAssignmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListIdentityAssignments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/ListIdentityAssignments',
            authorizer_dot_payload__pb2.ListIdentityAssignmentsRequest.SerializeToString,
            authorizer_dot_payload__pb2.ListIdentityAssignmentsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPolicyAssignment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/GetPolicyAssignment',
            authorizer_dot_payload__pb2.GetPolicyAssignmentRequest.SerializeToString,
            authorizer_dot_payload__pb2.GetPolicyAssignmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreateResource',
            authorizer_dot_payload__pb2.CreateResourceRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreateResourceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeleteResource',
            authorizer_dot_payload__pb2.DeleteResourceRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeleteResourceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateOrganization(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreateOrganization',
            authorizer_dot_payload__pb2.CreateOrganizationRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreateOrganizationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOrganization(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/GetOrganization',
            authorizer_dot_payload__pb2.GetOrganizationRequest.SerializeToString,
            authorizer_dot_payload__pb2.GetOrganizationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteOrganization(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeleteOrganization',
            authorizer_dot_payload__pb2.DeleteOrganizationRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeleteOrganizationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreatePolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreatePolicy',
            authorizer_dot_payload__pb2.CreatePolicyRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreatePolicyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/GetPolicy',
            authorizer_dot_payload__pb2.GetPolicyRequest.SerializeToString,
            authorizer_dot_payload__pb2.GetPolicyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeletePolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeletePolicy',
            authorizer_dot_payload__pb2.DeletePolicyRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeletePolicyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListPolicies(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/ListPolicies',
            authorizer_dot_payload__pb2.ListPoliciesRequest.SerializeToString,
            authorizer_dot_payload__pb2.ListPoliciesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreatePolicyBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreatePolicyBinding',
            authorizer_dot_payload__pb2.CreatePolicyBindingRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreatePolicyBindingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeletePolicyBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeletePolicyBinding',
            authorizer_dot_payload__pb2.DeletePolicyBindingRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeletePolicyBindingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateRole(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreateRole',
            authorizer_dot_payload__pb2.CreateRoleRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreateRoleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRole(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/GetRole',
            authorizer_dot_payload__pb2.GetRoleRequest.SerializeToString,
            authorizer_dot_payload__pb2.GetRoleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteRole(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeleteRole',
            authorizer_dot_payload__pb2.DeleteRoleRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeleteRoleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListRoles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/ListRoles',
            authorizer_dot_payload__pb2.ListRolesRequest.SerializeToString,
            authorizer_dot_payload__pb2.ListRolesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateRoleAction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/CreateRoleAction',
            authorizer_dot_payload__pb2.CreateRoleActionRequest.SerializeToString,
            authorizer_dot_payload__pb2.CreateRoleActionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteRoleAction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.authorizer.AuthorizerService/DeleteRoleAction',
            authorizer_dot_payload__pb2.DeleteRoleActionRequest.SerializeToString,
            authorizer_dot_payload__pb2.DeleteRoleActionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import rich.markup
import rich_click as click
from rich.console import Console

from union._config import _DEFAULT_DOMAIN, _DEFAULT_PROJECT_BYOC
from union.remote import UnionRemote


@click.group()
def stop():
    """Stop a resource."""


@stop.command()
@click.option("--name", type=str, required=True)
@click.option("--project", default=_DEFAULT_PROJECT_BYOC, help="Project name")
@click.option("--domain", default=_DEFAULT_DOMAIN, help="Domain name")
def apps(name: str, project: str, domain: str):
    app_remote = UnionRemote(default_domain=domain, default_project=project)
    app_remote.stop_app(name=name)


@stop.command()
@click.option("--name", type=str, required=True)
@click.option("--project", default=_DEFAULT_PROJECT_BYOC, help="Project name")
@click.option("--domain", default=_DEFAULT_DOMAIN, help="Domain name")
def workspace(name: str, project: str, domain: str):
    from union._async import run_sync
    from union.configuration import UnionAIPlugin
    from union.workspace._vscode_remote import WorkspaceRemote

    remote = UnionAIPlugin.get_remote(config=None, project=project, domain=domain)
    ws_remote = WorkspaceRemote(default_project=project, default_domain=domain, union_remote=remote)
    console = Console()
    try:
        response = run_sync(ws_remote.stop_workspace_instance, workspace_definition_name=name)
    except Exception as exc:
        error_msg = exc.details() if hasattr(exc, "details") else str(exc)
        console.print(rich.markup.escape(f"Error stopping workspace instance '{name}': {error_msg}"), style="red")
        raise exc
    else:
        console.print(f"Workspace instance stopped: {response}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/deployment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x63ommon/deployment.proto\x12\x0f\x63loudidl.common*7\n\nDeployment\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04\x42YOC\x10\x01\x12\x0e\n\nSERVERLESS\x10\x02\x42\xb0\x01\n\x13\x63om.cloudidl.commonB\x0f\x44\x65ploymentProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/common\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43loudidl.Common\xca\x02\x0f\x43loudidl\\Common\xe2\x02\x1b\x43loudidl\\Common\\GPBMetadata\xea\x02\x10\x43loudidl::Commonb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.deployment_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.commonB\017DeploymentProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/common\242\002\003CCX\252\002\017Cloudidl.Common\312\002\017Cloudidl\\Common\342\002\033Cloudidl\\Common\\GPBMetadata\352\002\020Cloudidl::Common'
  _globals['_DEPLOYMENT']._serialized_start=44
  _globals['_DEPLOYMENT']._serialized_end=99
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identity_pb2 as common_dot_identity__pb2
from flyteidl.admin import task_pb2 as flyteidl_dot_admin_dot_task__pb2
from flyteidl.core import identifier_pb2 as flyteidl_dot_core_dot_identifier__pb2
from flyteidl.core import security_pb2 as flyteidl_dot_core_dot_security__pb2
from flyteidl.core import tasks_pb2 as flyteidl_dot_core_dot_tasks__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$workspace/workspace_definition.proto\x12\x12\x63loudidl.workspace\x1a\x15\x63ommon/identity.proto\x1a\x19\x66lyteidl/admin/task.proto\x1a\x1e\x66lyteidl/core/identifier.proto\x1a\x1c\x66lyteidl/core/security.proto\x1a\x19\x66lyteidl/core/tasks.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\"\xb5\x01\n\x1dWorkspaceDefinitionIdentifier\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x1b\n\x04name\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12!\n\x07version\x18\x05 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07version\"\xa3\x01\n\x13WorkspaceDefinition\x12K\n\x02id\x18\x01 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12?\n\x04spec\x18\x02 \x01(\x0b\x32+.cloudidl.workspace.WorkspaceDefinitionSpecR\x04spec\"\xad\x07\n\x17WorkspaceDefinitionSpec\x12/\n\x11short_description\x18\x01 \x01(\tB\x02\x18\x01R\x10shortDescription\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x32\n\x07task_id\x18\x04 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\x06taskId\x12\x46\n\x05state\x18\x05 \x01(\x0e\x32,.cloudidl.workspace.WorkspaceDefinitionStateB\x02\x18\x01R\x05state\x12\x39\n\ttask_spec\x18\x06 \x01(\x0b\x32\x18.flyteidl.admin.TaskSpecB\x02\x18\x01R\x08taskSpec\x12\x36\n\tresources\x18\x07 \x01(\x0b\x32\x18.flyteidl.core.ResourcesR\tresources\x12O\n\x12\x65xtended_resources\x18\x08 \x01(\x0b\x32 .flyteidl.core.ExtendedResourcesR\x11\x65xtendedResources\x12\x1f\n\x0bttl_seconds\x18\t \x01(\x04R\nttlSeconds\x12I\n\x10security_context\x18\n \x01(\x0b\x32\x1e.flyteidl.core.SecurityContextR\x0fsecurityContext\x12X\n\x12python_environment\x18\x0b \x01(\x0b\x32%.cloudidl.workspace.PythonEnvironmentB\x02\x18\x01R\x11pythonEnvironment\x12U\n\x11\x63onda_environment\x18\x0c \x01(\x0b\x32$.cloudidl.workspace.CondaEnvironmentB\x02\x18\x01R\x10\x63ondaEnvironment\x12\x1d\n\non_startup\x18\r \x03(\tR\tonStartup\x12\'\n\x0f\x63ontainer_image\x18\x0e \x01(\tR\x0e\x63ontainerImage\x12%\n\x0eworkspace_root\x18\x0f \x01(\tR\rworkspaceRoot\x12\x1f\n\x0bworking_dir\x18\x10 \x01(\tR\nworkingDir\"3\n\x11PythonEnvironment\x12\x1a\n\x08packages\x18\x01 \x03(\tR\x08packages:\x02\x18\x01\"N\n\x10\x43ondaEnvironment\x12\x1a\n\x08packages\x18\x01 \x03(\tR\x08packages\x12\x1a\n\x08\x63hannels\x18\x02 \x03(\tR\x08\x63hannels:\x02\x18\x01\"\x90\x01\n\x1bWorkspaceDefinitionEntityId\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x1b\n\x04name\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"\x89\x02\n\rAuditMetadata\x12\x39\n\ncreated_at\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12@\n\ncreated_by\x18\x03 \x01(\x0b\x32!.cloudidl.common.EnrichedIdentityR\tcreatedBy\x12@\n\nupdated_by\x18\x04 \x01(\x0b\x32!.cloudidl.common.EnrichedIdentityR\tupdatedBy\"\xa1\x02\n\x19WorkspaceDefinitionEntity\x12I\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceDefinitionEntityIdB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x42\n\x05state\x18\x02 \x01(\x0e\x32,.cloudidl.workspace.WorkspaceDefinitionStateR\x05state\x12+\n\x11short_description\x18\x03 \x01(\tR\x10shortDescription\x12H\n\x0e\x61udit_metadata\x18\x04 \x01(\x0b\x32!.cloudidl.workspace.AuditMetadataR\rauditMetadata*\x96\x01\n\x18WorkspaceDefinitionState\x12*\n&WORKSPACE_DEFINITION_STATE_UNSPECIFIED\x10\x00\x12%\n!WORKSPACE_DEFINITION_STATE_ACTIVE\x10\x01\x12\'\n#WORKSPACE_DEFINITION_STATE_ARCHIVED\x10\x02\x42\xcb\x01\n\x16\x63om.cloudidl.workspaceB\x18WorkspaceDefinitionProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\030WorkspaceDefinitionProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['project']._options = None
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['domain']._options = None
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['name']._options = None
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['version']._options = None
  _WORKSPACEDEFINITIONIDENTIFIER.fields_by_name['version']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITION.fields_by_name['id']._options = None
  _WORKSPACEDEFINITION.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WORKSPACEDEFINITIONSPEC.fields_by_name['short_description']._options = None
  _WORKSPACEDEFINITIONSPEC.fields_by_name['short_description']._serialized_options = b'\030\001'
  _WORKSPACEDEFINITIONSPEC.fields_by_name['state']._options = None
  _WORKSPACEDEFINITIONSPEC.fields_by_name['state']._serialized_options = b'\030\001'
  _WORKSPACEDEFINITIONSPEC.fields_by_name['task_spec']._options = None
  _WORKSPACEDEFINITIONSPEC.fields_by_name['task_spec']._serialized_options = b'\030\001'
  _WORKSPACEDEFINITIONSPEC.fields_by_name['python_environment']._options = None
  _WORKSPACEDEFINITIONSPEC.fields_by_name['python_environment']._serialized_options = b'\030\001'
  _WORKSPACEDEFINITIONSPEC.fields_by_name['conda_environment']._options = None
  _WORKSPACEDEFINITIONSPEC.fields_by_name['conda_environment']._serialized_options = b'\030\001'
  _PYTHONENVIRONMENT._options = None
  _PYTHONENVIRONMENT._serialized_options = b'\030\001'
  _CONDAENVIRONMENT._options = None
  _CONDAENVIRONMENT._serialized_options = b'\030\001'
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['project']._options = None
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['domain']._options = None
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['name']._options = None
  _WORKSPACEDEFINITIONENTITYID.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEDEFINITIONENTITY.fields_by_name['id']._options = None
  _WORKSPACEDEFINITIONENTITY.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_WORKSPACEDEFINITIONSTATE']._serialized_start=2392
  _globals['_WORKSPACEDEFINITIONSTATE']._serialized_end=2542
  _globals['_WORKSPACEDEFINITIONIDENTIFIER']._serialized_start=258
  _globals['_WORKSPACEDEFINITIONIDENTIFIER']._serialized_end=439
  _globals['_WORKSPACEDEFINITION']._serialized_start=442
  _globals['_WORKSPACEDEFINITION']._serialized_end=605
  _globals['_WORKSPACEDEFINITIONSPEC']._serialized_start=608
  _globals['_WORKSPACEDEFINITIONSPEC']._serialized_end=1549
  _globals['_PYTHONENVIRONMENT']._serialized_start=1551
  _globals['_PYTHONENVIRONMENT']._serialized_end=1602
  _globals['_CONDAENVIRONMENT']._serialized_start=1604
  _globals['_CONDAENVIRONMENT']._serialized_end=1682
  _globals['_WORKSPACEDEFINITIONENTITYID']._serialized_start=1685
  _globals['_WORKSPACEDEFINITIONENTITYID']._serialized_end=1829
  _globals['_AUDITMETADATA']._serialized_start=1832
  _globals['_AUDITMETADATA']._serialized_end=2097
  _globals['_WORKSPACEDEFINITIONENTITY']._serialized_start=2100
  _globals['_WORKSPACEDEFINITIONENTITY']._serialized_end=2389
# @@protoc_insertion_point(module_scope)



from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import policy_pb2 as _policy_pb2
from union.internal.common import role_pb2 as _role_pb2
from union.internal.validate.validate import validate_pb2 as _validate_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class User(_message.Message):
    __slots__ = ["id", "spec", "roles", "policies"]
    ID_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    POLICIES_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.UserIdentifier
    spec: UserSpec
    roles: _containers.RepeatedCompositeFieldContainer[_role_pb2.Role]
    policies: _containers.RepeatedCompositeFieldContainer[_policy_pb2.Policy]
    def __init__(self, id: _Optional[_Union[_identifier_pb2.UserIdentifier, _Mapping]] = ..., spec: _Optional[_Union[UserSpec, _Mapping]] = ..., roles: _Optional[_Iterable[_Union[_role_pb2.Role, _Mapping]]] = ..., policies: _Optional[_Iterable[_Union[_policy_pb2.Policy, _Mapping]]] = ...) -> None: ...

class UserSpec(_message.Message):
    __slots__ = ["first_name", "last_name", "email", "organization", "user_handle", "groups", "photo_url"]
    FIRST_NAME_FIELD_NUMBER: _ClassVar[int]
    LAST_NAME_FIELD_NUMBER: _ClassVar[int]
    EMAIL_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    USER_HANDLE_FIELD_NUMBER: _ClassVar[int]
    GROUPS_FIELD_NUMBER: _ClassVar[int]
    PHOTO_URL_FIELD_NUMBER: _ClassVar[int]
    first_name: str
    last_name: str
    email: str
    organization: str
    user_handle: str
    groups: _containers.RepeatedScalarFieldContainer[str]
    photo_url: str
    def __init__(self, first_name: _Optional[str] = ..., last_name: _Optional[str] = ..., email: _Optional[str] = ..., organization: _Optional[str] = ..., user_handle: _Optional[str] = ..., groups: _Optional[_Iterable[str]] = ..., photo_url: _Optional[str] = ...) -> None: ...

class Application(_message.Message):
    __slots__ = ["id", "spec"]
    ID_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.ApplicationIdentifier
    spec: AppSpec
    def __init__(self, id: _Optional[_Union[_identifier_pb2.ApplicationIdentifier, _Mapping]] = ..., spec: _Optional[_Union[AppSpec, _Mapping]] = ...) -> None: ...

class AppSpec(_message.Message):
    __slots__ = ["name", "organization"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    name: str
    organization: str
    def __init__(self, name: _Optional[str] = ..., organization: _Optional[str] = ...) -> None: ...

class EnrichedIdentity(_message.Message):
    __slots__ = ["user", "application"]
    USER_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_FIELD_NUMBER: _ClassVar[int]
    user: User
    application: Application
    def __init__(self, user: _Optional[_Union[User, _Mapping]] = ..., application: _Optional[_Union[Application, _Mapping]] = ...) -> None: ...

class Identity(_message.Message):
    __slots__ = ["user_id", "application_id"]
    USER_ID_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    user_id: _identifier_pb2.UserIdentifier
    application_id: _identifier_pb2.ApplicationIdentifier
    def __init__(self, user_id: _Optional[_Union[_identifier_pb2.UserIdentifier, _Mapping]] = ..., application_id: _Optional[_Union[_identifier_pb2.ApplicationIdentifier, _Mapping]] = ...) -> None: ...
